from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from datetime import datetime
from typing import (
    Any,
    List,
    Literal,
)
from uuid import UUID

AnyValue = Any

class CertificateBlockListEntryInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')

class CertificateBlockListPageResponse(ObjectModel):
    certificateBlockListEntryInformation: List[CertificateBlockListEntryInformation] = ApiField(
        alias='certificateBlockListEntryInformation', default_factory=list
    )
    nextPage: str = ApiField(alias='nextPage')

class CertificateBlockListRequest(ObjectModel):
    fingerprints: List[str] = ApiField(alias='fingerprints', default_factory=list)

class CertificateBlockListResponse(ObjectModel):
    certificateBlockListEntryInformation: List[CertificateBlockListEntryInformation] = ApiField(
        alias='certificateBlockListEntryInformation', default_factory=list
    )

class CertificateImportInformation(ObjectModel):
    certificate: str = ApiField(alias='certificate')
    dekEncryptedPassword: str = ApiField(alias='dekEncryptedPassword')
    dekEncryptedPrivateKey: str = ApiField(alias='dekEncryptedPrivateKey')
    passwordEncryptedPrivateKey: str = ApiField(alias='passwordEncryptedPrivateKey')
    pkcs12Keystore: str = ApiField(alias='pkcs12Keystore')

class CertificateImportRequest(ObjectModel):
    edgeInstanceId: UUID = ApiField(alias='edgeInstanceId')
    encryptionKeyId: str = ApiField(alias='encryptionKeyId')
    importInformation: List[CertificateImportInformation] = ApiField(alias='importInformation', default_factory=list)

class CertificateImportResponse(ObjectModel):
    creationDate: datetime = ApiField(alias='creationDate')
    id: str = ApiField(alias='id')

class CertificateImportStatusDetail(ObjectModel):
    certificateBytes: str = ApiField(alias='certificateBytes')
    fingerprint: str = ApiField(alias='fingerprint')
    reason: str = ApiField(alias='reason')
    status: Literal['FAILED', 'IMPORTED', 'PROCESSING', 'SKIPPED'] = ApiField(alias='status')

class CertificateImportStatusDetailResponse(ObjectModel):
    id: UUID = ApiField(alias='id')
    results: List[CertificateImportStatusDetail] = ApiField(alias='results', default_factory=list)
    status: Literal['COMPLETED', 'FAILED', 'PROCESSING', 'WAITING'] = ApiField(alias='status')

class CertificateInformation(ObjectModel):
    chainValidationStatus: List[
        Literal['CHAIN_BUILDING_FAILED', 'CHAIN_EXPIRE_BEFORE_EE', 'DISTRUSTED', 'INCOMPLETE_CHAIN',
        'OK', 'SELF_SIGNED', 'UNKNOWN_ERROR']] = ApiField(alias='chainValidationStatus', default_factory=list)
    encryptionType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(
        alias='encryptionType'
    )
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    issuerCertificateIds: List[UUID] = ApiField(alias='issuerCertificateIds', default_factory=list)
    keyStrength: int = ApiField(alias='keyStrength')
    signatureAlgorithm: Literal[
        'EC_DSA_WITH_SHA1', 'EC_DSA_WITH_SHA224', 'EC_DSA_WITH_SHA256', 'EC_DSA_WITH_SHA384', 'EC_DSA_WITH_SHA512', 'GOST_R3411_94_WITH_GOST_R3410_2001', 'GOST_R3411_94_WITH_GOST_R3410_94', 'ID_DSA_WITH_SHA1', 'MD2_WITH_RSA_ENCRYPTION',
        'MD5_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION2', 'SHA1_WITH_RSAandMGF1', 'SHA256_WITH_RSA_ENCRYPTION', 'SHA384_WITH_RSA_ENCRYPTION', 'SHA512_WITH_RSA_ENCRYPTION', 'UNKNOWN', 'dsaWithSHA1'] = ApiField(
        alias='signatureAlgorithm'
    )
    signatureHashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='signatureHashAlgorithm')
    subjectDN: str = ApiField(alias='subjectDN')
    validityEnd: datetime = ApiField(alias='validityEnd')
    validityStart: datetime = ApiField(alias='validityStart')

class CompanyDomainInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    domain: str = ApiField(alias='domain')
    emailSendDate: datetime = ApiField(alias='emailSendDate')
    id: UUID = ApiField(alias='id')
    productEntitlements: List[Literal['ANY', 'CODESIGN', 'DEVOPS', 'MIRA', 'OUTAGE_DETECTION']
    ] = ApiField(alias='productEntitlements', default_factory=list)
    status: Literal['AUTHORIZED', 'EXPIRED', 'PENDING'] = ApiField(alias='status')
    userId: UUID = ApiField(alias='userId')
    validationRetriesRemaining: int = ApiField(alias='validationRetriesRemaining')

class CompanyDomainRequest(ObjectModel):
    domains: List[str] = ApiField(alias='domains', default_factory=list)
    status: Literal['AUTHORIZED', 'EXPIRED', 'PENDING'] = ApiField(alias='status')

class CompanyDomainResponse(ObjectModel):
    companyDomains: List[CompanyDomainInformation] = ApiField(alias='companyDomains', default_factory=list)

class CompanyProperty(ObjectModel):
    propertyType: str = ApiField(alias='propertyType')

class CompanyPropertyData(ObjectModel):
    propertyType: str = ApiField(alias='propertyType')

class CompanyPropertyInformation(ObjectModel):
    id: UUID = ApiField(alias='id')
    type: Literal['CERTIFICATE_RETIREMENT', 'PROPERTY_1', 'PROPERTY_2'] = ApiField(alias='type')
    value: CompanyProperty = ApiField(alias='value')

class CompanyPropertyRequest(ObjectModel):
    type: Literal['CERTIFICATE_RETIREMENT', 'PROPERTY_1', 'PROPERTY_2'] = ApiField(alias='type')
    value: CompanyPropertyData = ApiField(alias='value')

class CompanyPropertyResponse(ObjectModel):
    properties: List[CompanyPropertyInformation] = ApiField(alias='properties', default_factory=list)

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class ExtendedTrustedCACertificateInformation(ObjectModel):
    authorityKeyIdentifierHash: str = ApiField(alias='authorityKeyIdentifierHash')
    certificatePolicies: List[TrustedCACertificatePolicyInformation] = ApiField(
        alias='certificatePolicies',
        default_factory=list
    )
    certificateType: Literal['CA', 'CROSS_CA', 'END_ENTITY', 'END_ENTITY_AC', 'ROOT_CA', 'SELF_ISSUED_CA'] = ApiField(
        alias='certificateType'
    )
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    crlDistributionPoints: List[str] = ApiField(alias='crlDistributionPoints', default_factory=list)
    encryptionType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(
        alias='encryptionType'
    )
    extendedKeyUsage: List[str] = ApiField(alias='extendedKeyUsage', default_factory=list)
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    issuerCertificateIds: List[UUID] = ApiField(alias='issuerCertificateIds', default_factory=list)
    issuerCertificates: List[CertificateInformation] = ApiField(alias='issuerCertificates', default_factory=list)
    issuerDN: str = ApiField(alias='issuerDN')
    issuerUrl: str = ApiField(alias='issuerUrl')
    keyStrength: int = ApiField(alias='keyStrength')
    keyUsage: List[str] = ApiField(alias='keyUsage', default_factory=list)
    ocspUrl: str = ApiField(alias='ocspUrl')
    pathLength: int = ApiField(alias='pathLength')
    serialNumber: str = ApiField(alias='serialNumber')
    signatureAlgorithm: Literal[
        'EC_DSA_WITH_SHA1', 'EC_DSA_WITH_SHA224', 'EC_DSA_WITH_SHA256', 'EC_DSA_WITH_SHA384', 'EC_DSA_WITH_SHA512', 'GOST_R3411_94_WITH_GOST_R3410_2001', 'GOST_R3411_94_WITH_GOST_R3410_94', 'ID_DSA_WITH_SHA1', 'MD2_WITH_RSA_ENCRYPTION',
        'MD5_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION2', 'SHA1_WITH_RSAandMGF1', 'SHA256_WITH_RSA_ENCRYPTION', 'SHA384_WITH_RSA_ENCRYPTION', 'SHA512_WITH_RSA_ENCRYPTION', 'UNKNOWN', 'dsaWithSHA1'] = ApiField(
        alias='signatureAlgorithm'
    )
    signatureHashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='signatureHashAlgorithm')
    source: Literal['GLOBALLY_TRUSTED', 'USER_PROVIDED'] = ApiField(alias='source')
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectCN: List[str] = ApiField(alias='subjectCN', default_factory=list)
    subjectDN: str = ApiField(alias='subjectDN')
    subjectKeyIdentifierHash: str = ApiField(alias='subjectKeyIdentifierHash')
    validityEnd: datetime = ApiField(alias='validityEnd')
    validityStart: datetime = ApiField(alias='validityStart')
    version: int = ApiField(alias='version')

class GeneralNamesData(ObjectModel):
    dNSName: List[str] = ApiField(alias='dNSName', default_factory=list)
    directoryName: List[str] = ApiField(alias='directoryName', default_factory=list)
    ediPartyName: List[str] = ApiField(alias='ediPartyName', default_factory=list)
    iPAddress: List[str] = ApiField(alias='iPAddress', default_factory=list)
    otherName: List[str] = ApiField(alias='otherName', default_factory=list)
    registeredID: List[str] = ApiField(alias='registeredID', default_factory=list)
    rfc822Name: List[str] = ApiField(alias='rfc822Name', default_factory=list)
    uniformResourceIdentifier: List[str] = ApiField(alias='uniformResourceIdentifier', default_factory=list)
    x400Address: List[str] = ApiField(alias='x400Address', default_factory=list)

class TrustedCACertificateDeletionRequest(ObjectModel):
    fingerprints: List[str] = ApiField(alias='fingerprints', default_factory=list)

class TrustedCACertificateInformation(ObjectModel):
    authorityKeyIdentifierHash: str = ApiField(alias='authorityKeyIdentifierHash')
    certificatePolicies: List[TrustedCACertificatePolicyInformation] = ApiField(
        alias='certificatePolicies',
        default_factory=list
    )
    certificateType: Literal['CA', 'CROSS_CA', 'END_ENTITY', 'END_ENTITY_AC', 'ROOT_CA', 'SELF_ISSUED_CA'] = ApiField(
        alias='certificateType'
    )
    companyId: UUID = ApiField(alias='companyId')
    creationDate: datetime = ApiField(alias='creationDate')
    crlDistributionPoints: List[str] = ApiField(alias='crlDistributionPoints', default_factory=list)
    encryptionType: Literal['DSA', 'EC', 'ECGOST3410', 'GOST3410', 'RESERVED3', 'RSA', 'UNKNOWN'] = ApiField(
        alias='encryptionType'
    )
    extendedKeyUsage: List[str] = ApiField(alias='extendedKeyUsage', default_factory=list)
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    issuerCertificateIds: List[UUID] = ApiField(alias='issuerCertificateIds', default_factory=list)
    issuerDN: str = ApiField(alias='issuerDN')
    issuerUrl: str = ApiField(alias='issuerUrl')
    keyStrength: int = ApiField(alias='keyStrength')
    keyUsage: List[str] = ApiField(alias='keyUsage', default_factory=list)
    ocspUrl: str = ApiField(alias='ocspUrl')
    pathLength: int = ApiField(alias='pathLength')
    serialNumber: str = ApiField(alias='serialNumber')
    signatureAlgorithm: Literal[
        'EC_DSA_WITH_SHA1', 'EC_DSA_WITH_SHA224', 'EC_DSA_WITH_SHA256', 'EC_DSA_WITH_SHA384', 'EC_DSA_WITH_SHA512', 'GOST_R3411_94_WITH_GOST_R3410_2001', 'GOST_R3411_94_WITH_GOST_R3410_94', 'ID_DSA_WITH_SHA1', 'MD2_WITH_RSA_ENCRYPTION',
        'MD5_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION', 'SHA1_WITH_RSA_ENCRYPTION2', 'SHA1_WITH_RSAandMGF1', 'SHA256_WITH_RSA_ENCRYPTION', 'SHA384_WITH_RSA_ENCRYPTION', 'SHA512_WITH_RSA_ENCRYPTION', 'UNKNOWN', 'dsaWithSHA1'] = ApiField(
        alias='signatureAlgorithm'
    )
    signatureHashAlgorithm: Literal['GOSTR3411_94', 'MD2', 'MD5', 'SHA1', 'SHA224',
    'SHA256', 'SHA384', 'SHA512', 'UNKNOWN'] = ApiField(alias='signatureHashAlgorithm')
    source: Literal['GLOBALLY_TRUSTED', 'USER_PROVIDED'] = ApiField(alias='source')
    subjectAlternativeNamesByType: GeneralNamesData = ApiField(alias='subjectAlternativeNamesByType')
    subjectCN: List[str] = ApiField(alias='subjectCN', default_factory=list)
    subjectDN: str = ApiField(alias='subjectDN')
    subjectKeyIdentifierHash: str = ApiField(alias='subjectKeyIdentifierHash')
    validityEnd: datetime = ApiField(alias='validityEnd')
    validityStart: datetime = ApiField(alias='validityStart')
    version: int = ApiField(alias='version')

class TrustedCACertificatePolicyInformation(ObjectModel):
    cps: str = ApiField(alias='cps')
    userNotice: List[str] = ApiField(alias='userNotice', default_factory=list)

class TrustedCACertificateResponse(ObjectModel):
    certificates: List[TrustedCACertificateInformation] = ApiField(alias='certificates', default_factory=list)
    certificatesCount: int = ApiField(alias='certificatesCount')

class TrustedCACertificatesRequest(ObjectModel):
    encodedCertificates: List[str] = ApiField(alias='encodedCertificates', default_factory=list)

class CertificateRetirementCompanyProperty(CompanyProperty):
    accessRole: Literal['ADMIN', 'EVERYONE'] = ApiField(alias='accessRole')
    custom: bool = ApiField(alias='custom')
    period: int = ApiField(alias='period')

class CertificateRetirementCompanyPropertyData(CompanyPropertyData):
    accessRole: Literal['ADMIN', 'EVERYONE'] = ApiField(alias='accessRole')
    custom: bool = ApiField(alias='custom')
    period: int = ApiField(alias='period')

CertificateBlockListEntryInformation.update_forward_refs()
CertificateBlockListPageResponse.update_forward_refs()
CertificateBlockListRequest.update_forward_refs()
CertificateBlockListResponse.update_forward_refs()
CertificateImportInformation.update_forward_refs()
CertificateImportRequest.update_forward_refs()
CertificateImportResponse.update_forward_refs()
CertificateImportStatusDetail.update_forward_refs()
CertificateImportStatusDetailResponse.update_forward_refs()
CertificateInformation.update_forward_refs()
CertificateRetirementCompanyProperty.update_forward_refs()
CertificateRetirementCompanyPropertyData.update_forward_refs()
CompanyDomainInformation.update_forward_refs()
CompanyDomainRequest.update_forward_refs()
CompanyDomainResponse.update_forward_refs()
CompanyProperty.update_forward_refs()
CompanyPropertyData.update_forward_refs()
CompanyPropertyInformation.update_forward_refs()
CompanyPropertyRequest.update_forward_refs()
CompanyPropertyResponse.update_forward_refs()
ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
ExtendedTrustedCACertificateInformation.update_forward_refs()
GeneralNamesData.update_forward_refs()
TrustedCACertificateDeletionRequest.update_forward_refs()
TrustedCACertificateInformation.update_forward_refs()
TrustedCACertificatePolicyInformation.update_forward_refs()
TrustedCACertificateResponse.update_forward_refs()
TrustedCACertificatesRequest.update_forward_refs()



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.top import TopAttributes

class X509RootCertificateContainerAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "X509 Root Certificate Container"



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes
from pyvenafi.tpp.attributes.proxy import ProxyAttributes

class RedhatCAAttributes(HTTPCABaseAttributes, ProxyAttributes, metaclass=IterableMeta):
    __config_class__ = "Redhat CA"
    agent_port = Attribute('Agent Port', min_version='21.4')
    agent_url_surffix = Attribute('Agent URL Surffix', min_version='21.4')
    end_entity_port = Attribute('End Entity Port', min_version='21.4')
    end_entity_url_surffix = Attribute('End Entity URL Surffix', min_version='21.4')
    use_profile = Attribute('Use Profile', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class WorkflowRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Workflow Root"
    approval_reason = Attribute('Approval Reason', min_version='21.4')
    approver_not_found_expiration = Attribute('Approver Not Found Expiration', min_version='21.4')
    expiration_days = Attribute('Expiration Days', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class VamCaviumAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "VamCavium"
    cavium_utility_path = Attribute('Cavium Utility Path')
    key_list_path = Attribute('Key List Path')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_trust_store_base import CertificateTrustStoreBaseAttributes

class PaloAltoNetworkFWTrustStoreAttributes(CertificateTrustStoreBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Palo Alto Network FW Trust Store"
    lock_config = Attribute('Lock Config', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.certificate_trust_store_base import CertificateTrustStoreBaseAttributes

class BlueCoatSSLVATrustStoreAttributes(CertificateTrustStoreBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "BlueCoat SSLVA Trust Store"
    create_lists = Attribute('Create Lists', min_version='21.4')
    key_store = Attribute('Key Store', min_version='21.4')



from __future__ import annotations

class DN(str):
    def __init__(self, dn: str):
        super().__init__()
        self.dn = str(dn)
        if '\\' in self.dn:
            self._parent, self.name = self.dn.rsplit('\\', maxsplit=1)
        else:
            self._parent, self.name = None, self.dn
        self.depth = self.dn.count('\\') - 1

    def __new__(cls, dn: str):
        return str.__new__(cls, dn)

    def __add__(self, other):
        return DN(self.dn + other)

    @property
    def parent(self):
        return DN(self._parent) if self._parent else None



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class UpgradeTaskAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Upgrade Task"
    display_name = Attribute('Display Name', min_version='21.4')
    upgrade_task_execution_order_id = Attribute('Upgrade Task Execution Order Id', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.flow_action_approver_base import FlowActionApproverBaseAttributes

class FlowActionDirectApproversAttributes(FlowActionApproverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Flow Action Direct Approvers"
    direct_approver = Attribute('Direct Approver', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class PaloAltoNetworkFWAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Palo Alto Network FW"
    certificate_only = Attribute('Certificate Only', min_version='21.4')
    chain_cert = Attribute('Chain Cert', min_version='21.4')
    create_decryption_policy = Attribute('Create Decryption Policy', min_version='21.4')
    decryption_destinations = Attribute('Decryption Destinations', min_version='21.4')
    decryption_policy = Attribute('Decryption Policy', min_version='21.4')
    decryption_profile = Attribute('Decryption Profile', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    lock_config = Attribute('Lock Config', min_version='21.4')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    replace_store = Attribute('Replace Store', min_version='21.4')
    tls_service_profile = Attribute('TLS Service Profile', min_version='21.4')
    temporary_name = Attribute('Temporary Name', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.log_channel import LogChannelAttributes

class LogSyslogAttributes(LogChannelAttributes, metaclass=IterableMeta):
    __config_class__ = "Log Syslog"
    credential = Attribute('Credential', min_version='21.4')
    enable_tls = Attribute('Enable TLS', min_version='21.4')
    facility = Attribute('Facility', min_version='21.4')
    message_format = Attribute('Message Format', min_version='21.4')
    message_prefix = Attribute('Message Prefix', min_version='21.4')
    port = Attribute('Port', min_version='21.4')
    protocol = Attribute('Protocol', min_version='21.4')
    target = Attribute('Target', min_version='21.4')
    timeout = Attribute('Timeout', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class UpgradeAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Upgrade"
    grouping_id = Attribute('Grouping Id', min_version='21.4')



from __future__ import annotations

from typing import (
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import (
    config,
    metadata,
)

class _Metadata(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Metadata')
        self.DefineItem = self._DefineItem(api_obj=self._api_obj, url=f'{self._url}/DefineItem')
        self.Find = self._Find(api_obj=self._api_obj, url=f'{self._url}/Find')
        self.FindItem = self._FindItem(api_obj=self._api_obj, url=f'{self._url}/FindItem')
        self.Get = self._Get(api_obj=self._api_obj, url=f'{self._url}/Get')
        self.GetItemGuids = self._GetItemGuids(api_obj=self._api_obj, url=f'{self._url}/GetItemGuids')
        self.GetItems = self._GetItems(api_obj=self._api_obj, url=f'{self._url}/GetItems')
        self.GetItemsForClass = self._GetItemsForClass(api_obj=self._api_obj, url=f'{self._url}/GetItemsForClass')
        self.GetPolicyItems = self._GetPolicyItems(api_obj=self._api_obj, url=f'{self._url}/GetPolicyItems')
        self.Items = self._Items(api_obj=self._api_obj, url=f'{self._url}/Items')
        self.LoadItem = self._LoadItem(api_obj=self._api_obj, url=f'{self._url}/LoadItem')
        self.LoadItemGuid = self._LoadItemGuid(api_obj=self._api_obj, url=f'{self._url}/LoadItemGuid')
        self.ReadEffectiveValues = self._ReadEffectiveValues(
            api_obj=self._api_obj,
            url=f'{self._url}/ReadEffectiveValues'
        )
        self.ReadPolicy = self._ReadPolicy(api_obj=self._api_obj, url=f'{self._url}/ReadPolicy')
        self.Set = self._Set(api_obj=self._api_obj, url=f'{self._url}/Set')
        self.SetPolicy = self._SetPolicy(api_obj=self._api_obj, url=f'{self._url}/SetPolicy')
        self.UndefineItem = self._UndefineItem(api_obj=self._api_obj, url=f'{self._url}/UndefineItem')
        self.UpdateItem = self._UpdateItem(api_obj=self._api_obj, url=f'{self._url}/UpdateItem')

    class _DefineItem(WebSdkEndpoint):
        def post(self, item: Union[dict, metadata.Item]):
            body = {
                'Item': item
            }

            class Output(WebSdkOutputModel):
                dn: str = ApiField(alias='DN')
                item: metadata.Item = ApiField(alias='Item')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Find(WebSdkEndpoint):
        def post(self, item: str = None, item_guid: str = None, value: str = None):
            body = {
                'Item'    : item,
                'ItemGuid': item_guid,
                'Value'   : value
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                objects: list[config.Object] = ApiField(default_factory=list, alias='Objects')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _FindItem(WebSdkEndpoint):
        def post(self, name: str):
            body = {
                'Name': name
            }

            class Output(WebSdkOutputModel):
                item_guid: str = ApiField(alias='ItemGuid')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Get(WebSdkEndpoint):
        def post(self, dn: str, all_included: bool = None):
            body = {
                'DN' : dn,
                'All': all_included
            }

            class Output(WebSdkOutputModel):
                data: metadata.Data = ApiField(alias='Data')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetItemGuids(WebSdkEndpoint):
        def post(self, dn: str):
            body = {
                'DN': dn
            }

            class Output(WebSdkOutputModel):
                item_guids: list[str] = ApiField(default_factory=list, alias='ItemGuids')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetItems(WebSdkEndpoint):
        def post(self, dn: str):
            body = {
                'DN': dn
            }

            class Output(WebSdkOutputModel):
                items: list[metadata.Item] = ApiField(default_factory=list, alias='Items')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetItemsForClass(WebSdkEndpoint):
        def post(self, config_class: str):
            body = {
                'ConfigClass': config_class
            }

            class Output(WebSdkOutputModel):
                items: list[metadata.Item] = ApiField(default_factory=list, alias='Items')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetPolicyItems(WebSdkEndpoint):
        def post(self, dn: str):
            body = {
                'DN': dn
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                policy_items: list[metadata.PolicyItem] = ApiField(default_factory=list, alias='PolicyItems')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Items(WebSdkEndpoint):
        def get(self):
            class Output(WebSdkOutputModel):
                items: list[metadata.Item] = ApiField(default_factory=list, alias='Items')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._get())

    class _LoadItem(WebSdkEndpoint):
        def post(self, dn: str):
            body = {
                'DN': dn
            }

            class Output(WebSdkOutputModel):
                item: metadata.Item = ApiField(alias='Item')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _LoadItemGuid(WebSdkEndpoint):
        def post(self, dn: str):
            body = {
                'DN': dn
            }

            class Output(WebSdkOutputModel):
                item_guid: str = ApiField(alias='ItemGuid')
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ReadEffectiveValues(WebSdkEndpoint):
        def post(self, dn: str, item_guid: str):
            body = {
                'DN'      : dn,
                'ItemGuid': item_guid
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                policy_dn: str = ApiField(alias='PolicyDn')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))
                values: list[str] = ApiField(default_factory=list, alias='Values')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ReadPolicy(WebSdkEndpoint):
        def post(self, dn: str, item_guid: str, obj_type: str):
            body = {
                'DN'      : dn,
                'ItemGuid': item_guid,
                'Type'    : obj_type
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))
                values: list[str] = ApiField(default_factory=list, alias='Values')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Set(WebSdkEndpoint):
        def post(self, dn: str, guid_data: list[metadata.GuidData], keep_existing: bool = False):
            body = {
                'DN'          : dn,
                'GuidData'    : guid_data,
                'KeepExisting': keep_existing
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _SetPolicy(WebSdkEndpoint):
        def post(self, dn: str, config_class: str, guid_data: list[metadata.GuidData], locked: bool = False):
            body = {
                'DN'         : dn,
                'ConfigClass': config_class,
                'GuidData'   : guid_data,
                'Locked'     : locked
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UndefineItem(WebSdkEndpoint):
        def post(self, item_guid: str, remove_data: bool = True):
            body = {
                'ItemGuid'  : item_guid,
                'RemoveData': remove_data
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UpdateItem(WebSdkEndpoint):
        def post(self, item: Union[dict, metadata.Item] = None, update: Union[dict, metadata.Update] = None):
            body = {
                'ItemGuid': item,
                'Update'  : update
            }

            class Output(WebSdkOutputModel):
                locked: bool = ApiField(alias='Locked')
                result: metadata.Result = ApiField(alias='Result', converter=lambda x: metadata.Result(code=x))

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from datetime import datetime
from typing import (
    Union,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    ObjectModel,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import certificate

class _Certificates(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Certificates')
        self.Associate = self._Associate(api_obj=api_obj, url=f'{self._url}/Associate')
        self.CheckPolicy = self._CheckPolicy(api_obj=api_obj, url=f'{self._url}/CheckPolicy')
        self.Dissociate = self._Dissociate(api_obj=api_obj, url=f'{self._url}/Dissociate')
        self.Import = self._Import(api_obj=api_obj, url=f'{self._url}/Import')
        self.Push = self._Push(api_obj=api_obj, url=f'{self._url}/Push')
        self.Renew = self._Renew(api_obj=api_obj, url=f'{self._url}/Renew')
        self.Request = self._Request(api_obj=api_obj, url=f'{self._url}/Request')
        self.Reset = self._Reset(api_obj=api_obj, url=f'{self._url}/Reset')
        self.Retrieve = self._Retrieve(api_obj=api_obj, url=f'{self._url}/Retrieve')
        self.Retry = self._Retry(api_obj=api_obj, url=f'{self._url}/Retry')
        self.Revoke = self._Revoke(api_obj=api_obj, url=f'{self._url}/Revoke')
        self.Validate = self._Validate(api_obj=api_obj, url=f'{self._url}/Validate')

    def Guid(self, guid):
        return self._Guid(api_obj=self._api_obj, url=f'{self._url}/{guid}')

    def get(
        self, limit: int = None, offset: int = None, optional_fields: list[str] = None,
        filters: Union[dict, certificate.CertificateFilter] = None
    ):
        params = {
            'Limit'         : limit,
            'Offset'        : offset,
            'OptionalFields': optional_fields
        }
        if isinstance(filters, dict):
            params.update(filters)
        elif isinstance(filters, ObjectModel):
            params.update(filters.dict(by_alias=True, exclude_none=True))

        class Output(WebSdkOutputModel):
            links: list[certificate.Link] = ApiField(default_factory=list, alias='_links')
            x_record_count: int = ApiField(alias='X-Record-Count')
            certificates: list[certificate.Certificate] = ApiField(default_factory=list, alias='Certificates')
            data_range: str = ApiField(alias='DataRange')
            total_count: int = ApiField(alias='TotalCount')

        return generate_output(response=self._get(params=params), output_cls=Output)

    def head(self, filters: Union[dict, certificate.CertificateFilter] = None):
        params = filters

        class Output(WebSdkOutputModel):
            @property
            def x_record_count(self):
                xrc = int(self.api_response.headers.get('X-Record-Count'))
                return xrc

        return generate_output(response=self._get(params=params), output_cls=Output)

    class _Associate(WebSdkEndpoint):
        def post(self, application_dn: str, certificate_dn: str, push_to_new: bool = False):
            body = {
                'CertificateDN': certificate_dn,
                'ApplicationDN': application_dn,
                'PushToNew'    : push_to_new
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _CheckPolicy(WebSdkEndpoint):
        def post(self, policy_dn: str, pkcs10: str = None):
            body = {
                'PolicyDN': policy_dn,
                'PKSC10'  : pkcs10
            }

            class Output(WebSdkOutputModel):
                csr: certificate.CSR = ApiField(alias='CSR')
                policy: certificate.Policy = ApiField(alias='Policy')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Dissociate(WebSdkEndpoint):
        def post(self, certificate_dn: str, application_dn: list[str], delete_orphans: bool = False):
            body = {
                'CertificateDN': certificate_dn,
                'ApplicationDN': application_dn,
                'DeleteOrphans': delete_orphans
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Guid(WebSdkEndpoint):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.PreviousVersions = self._PreviousVersions(api_obj=self._api_obj, url=f'{self._url}/PreviousVersions')
            self.ValidationResults = self._ValidationResults(
                api_obj=self._api_obj,
                url=f'{self._url}/ValidationResults'
            )

        def delete(self):
            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._delete(), output_cls=Output)

        def get(self):
            class Output(WebSdkOutputModel):
                approver: list[str] = ApiField(default_factory=list, alias='Approver')
                certificate_authority_dn: datetime = ApiField(alias='CertificateAuthorityDN')
                certificate_details: certificate.CertificateDetails = ApiField(alias='CertificateDetails')
                consumers: list[str] = ApiField(alias='Consumers', default_factory=list)
                contact: list[str] = ApiField(default_factory=list, alias='Contact')
                created_by: str = ApiField(alias='CreatedBy')
                created_on: datetime = ApiField(alias='CreatedOn')
                custom_fields: list[certificate.NameTypeValue] = ApiField(default_factory=list, alias='CustomFields')
                disabled: bool = ApiField(alias='Disabled')
                dn: str = ApiField(alias='DN')
                guid: str = ApiField(alias='Guid')
                management_type: str = ApiField(alias='ManagementType')
                name: str = ApiField(alias='Name')
                parent_dn: str = ApiField(alias='ParentDN')
                processing_details: certificate.ProcessingDetails = ApiField(alias='ProcessingDetails')
                renewal_details: certificate.RenewalDetails = ApiField(alias='RenewalDetails')
                schema_class: str = ApiField(alias='SchemaClass')
                success: str = ApiField(alias='Success')
                validation_details: certificate.ValidationDetails = ApiField(alias='ValidationDetails')

            return generate_output(response=self._get(), output_cls=Output)

        def put(self, attribute_data: list[dict]):
            body = {
                "AttributeData": attribute_data
            }

            class Output(WebSdkOutputModel):
                success: str = ApiField(alias='Success')

            return generate_output(response=self._put(data=body), output_cls=Output)

        class _PreviousVersions(WebSdkEndpoint):
            def get(self, exclude_expired: bool = False, exclude_revoked: bool = False):
                params = {
                    'ExcludeExpired': exclude_expired,
                    'ExcludeRevoked': exclude_revoked
                }

                class Output(WebSdkOutputModel):
                    success: str = ApiField(default='', alias='Success')
                    previous_versions: list[certificate.PreviousVersions] = ApiField(
                        default_factory=list, alias='PreviousVersions'
                    )

                return generate_output(response=self._get(params=params), output_cls=Output)

        class _ValidationResults(WebSdkEndpoint):
            def get(self):
                class Output(WebSdkOutputModel):
                    file: list[certificate.File] = ApiField(default_factory=list, alias='File')
                    ssl_tls: list[certificate.SslTls] = ApiField(default_factory=list, alias='SslTls')

                return generate_output(response=self._get(), output_cls=Output)

    class _Import(WebSdkEndpoint):
        def post(
            self, certificate_data: str, policy_dn: str, ca_specific_attributes: list[certificate.NameValue] = None,
            object_name: str = None,
            password: str = None, private_key_data: str = None, reconcile: bool = False
        ):
            body = {
                'CertificateData'     : certificate_data,
                'PolicyDN'            : policy_dn,
                'CASpecificAttributes': ca_specific_attributes,
                'ObjectName'          : object_name,
                'Password'            : password,
                'PrivateKeyData'      : private_key_data,
                'Reconcile'           : reconcile
            }

            class Output(WebSdkOutputModel):
                certificate_dn: str = ApiField(alias='CertificateDN')
                certificate_vault_id: int = ApiField(alias='CertificateVaultID')
                guid: str = ApiField(alias='Guid')
                private_key_vault_id: int = ApiField(alias='PrivateKeyVaultID')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Push(WebSdkEndpoint):
        def post(self, certificate_dn: str, application_dn: list[str] = None, push_to_all: bool = False):
            body = {
                'ApplicationDN': application_dn,
                'CertificateDN': certificate_dn,
                'PushToAll'    : push_to_all
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Renew(WebSdkEndpoint):
        # noinspection ALL
        def post(
            self,
            certificate_dn: str,
            pkcs10: str = None,
            reenable: bool = False,
            format: certificate.CertificateFormat = None,
            password: str = None,
            include_private_key: bool = None,
            include_chain: bool = None,
            friendly_name: str = None,
            root_first_order: bool = None,
            keystore_password: str = None,
            work_to_do_timeout: int = None
        ):
            body = {
                'CertificateDN'    : certificate_dn,
                'PKCS10'           : pkcs10,
                'Reenable'         : reenable,
                'Format'           : format,
                'Password'         : password,
                'IncludePrivateKey': include_private_key,
                'IncludeChain'     : include_chain,
                'FriendlyName'     : friendly_name,
                'RootFirstOrder'   : root_first_order,
                'KeystorePassword' : keystore_password,
                'WorkToDoTimeout'  : work_to_do_timeout
            }

            class Output(WebSdkOutputModel):
                certificate_data: str = ApiField(alias='CertificateData')
                certificate_dn: str = ApiField(alias='CertificateDN')
                filename: str = ApiField(alias='Filename')
                format: certificate.CertificateFormat = ApiField(alias='Format')
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Request(WebSdkEndpoint):
        def post(
            self,
            policy_dn: str,
            approvers: list[dict] = None,
            cadn: str = None,
            ca_specific_attributes: list[dict] = None,
            certificate_type: str = None,
            city: str = None,
            contacts: list[dict] = None,
            country: str = None,
            custom_fields: list[dict] = None,
            created_by: str = None,
            devices: list[dict] = None,
            disable_automatic_renewal: bool = False,
            elliptic_curve: str = None,
            format: certificate.CertificateFormat = None,
            friendly_name: str = None,
            include_private_key: bool = False,
            include_chain: bool = None,
            key_algorithm: str = None,
            key_bit_size: int = None,
            keystore_password: str = None,
            management_type: str = None,
            password: str = None,
            object_name: str = None,
            organization: str = None,
            organizational_unit: str = None,
            origin: str = None,
            pkcs10: str = None,
            reenable: bool = False,
            root_first_order: bool = None,
            set_work_todo: bool = True,
            state: str = None,
            subject: str = None,
            subject_alt_names: list[dict] = None,
            work_to_do_timeout: int = None
        ):
            body = {
                'Approvers'              : approvers,
                'CADN'                   : cadn,
                'CASpecificAttributes'   : ca_specific_attributes,
                'CertificateType'        : certificate_type,
                'City'                   : city,
                'Contacts'               : contacts,
                'Country'                : country,
                'CustomFields'           : custom_fields,
                'CreatedBy'              : created_by,
                'Devices'                : devices,
                'DisableAutomaticRenewal': disable_automatic_renewal,
                'EllipticCurve'          : elliptic_curve,
                'Format'                 : format,
                'FriendlyName'           : friendly_name,
                'IncludePrivateKey'      : include_private_key,
                'IncludeChain'           : include_chain,
                'KeystorePassword'       : keystore_password,
                'KeyAlgorithm'           : key_algorithm,
                'KeyBitSize'             : key_bit_size,
                'ManagementType'         : management_type,
                'ObjectName'             : object_name,
                'Origin'                 : origin,
                'Organization'           : organization,
                'OrganizationalUnit'     : organizational_unit,
                'Password'               : password,
                'PKCS10'                 : pkcs10,
                'PolicyDN'               : policy_dn,
                'Reenable'               : reenable,
                'RootFirstOrder'         : root_first_order,
                'SetWorkToDo'            : set_work_todo,
                'State'                  : state,
                'Subject'                : subject,
                'SubjectAltNames'        : subject_alt_names,
                'WorkToDoTimeout'        : work_to_do_timeout
            }

            class Output(WebSdkOutputModel):
                certificate_data: str = ApiField(alias='CertificateData')
                filename: str = ApiField(alias='Filename')
                format: certificate.CertificateFormat = ApiField(alias='Format')
                certificate_dn: str = ApiField(alias='CertificateDN')
                guid: str = ApiField(alias='Guid')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Reset(WebSdkEndpoint):
        def post(self, certificate_dn: str, restart: bool = False, work_to_do_timeout: int = None):
            body = {
                'CertificateDN'  : certificate_dn,
                'Restart'        : restart,
                'WorkToDoTimeout': work_to_do_timeout
            }

            class Output(WebSdkOutputModel):
                private_key_mismatch_reset_completed: bool = ApiField(alias='PrivateKeyMismatchResetCompleted')
                processing_reset_completed: bool = ApiField(alias='ProcessingResetCompleted')
                restart_completed: bool = ApiField(alias='RestartCompleted')
                revocation_reset_completed: bool = ApiField(alias='RevocationResetCompleted')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Retrieve(WebSdkEndpoint):
        # noinspection ALL
        def get(
            self,
            certificate_dn: str,
            format: certificate.CertificateFormat,
            friendly_name: str,
            include_chain: bool = False,
            include_private_key: bool = False,
            keystore_password: str = None,
            password: str = None,
            root_first_order: bool = False,
            work_to_do_timeout: int = None
        ):
            params = {
                'CertificateDN'    : certificate_dn,
                'Format'           : format,
                'FriendlyName'     : friendly_name,
                'IncludeChain'     : include_chain,
                'IncludePrivateKey': include_private_key,
                'KeystorePassword' : keystore_password,
                'Password'         : password,
                'RootFirstOrder'   : root_first_order,
                'WorkToDoTimeout'  : work_to_do_timeout
            }

            return generate_output(response=self._get(params=params), output_cls=WebSdkOutputModel)

        def post(
            self,
            certificate_dn: str,
            format: certificate.CertificateFormat,
            friendly_name: str,
            include_chain: bool = False,
            include_private_key: bool = False,
            keystore_password: str = None,
            password: str = None,
            root_first_order: bool = False,
            work_to_do_timeout: int = None
        ):
            body = {
                'CertificateDN'    : certificate_dn,
                'Format'           : format,
                'FriendlyName'     : friendly_name,
                'IncludeChain'     : include_chain,
                'IncludePrivateKey': include_private_key,
                'KeystorePassword' : keystore_password,
                'Password'         : password,
                'RootFirstOrder'   : root_first_order,
                'WorkToDoTimeout'  : work_to_do_timeout
            }

            class Output(WebSdkOutputModel):
                certificate_data: str = ApiField(alias='CertificateData')
                filename: str = ApiField(alias='Filename')
                format: certificate.CertificateFormat = ApiField(alias='Format')

            return generate_output(response=self._post(data=body), output_cls=Output)

        def VaultId(self, vault_id: int):
            return self._VaultId(api_obj=self._api_obj, url=f'{self._url}/{vault_id}')

        class _VaultId(WebSdkEndpoint):
            # noinspection ALL
            def get(
                self, format: certificate.CertificateFormat, friendly_name: str, include_chain: bool = False,
                include_private_key: bool = False, keystore_password: str = None, password: str = None,
                root_first_order: bool = False
            ):
                params = {
                    'Format'           : format,
                    'FriendlyName'     : friendly_name,
                    'IncludeChain'     : include_chain,
                    'IncludePrivateKey': include_private_key,
                    'KeystorePassword' : keystore_password,
                    'Password'         : password,
                    'RootFirstOrder'   : root_first_order
                }

                return generate_output(response=self._get(params=params), output_cls=WebSdkOutputModel)

            # noinspection ALL
            def post(
                self, format: certificate.CertificateFormat, friendly_name: str, include_chain: bool = False,
                include_private_key: bool = False, keystore_password: str = None, password: str = None,
                root_first_order: bool = False
            ):
                body = {
                    'Format'           : format,
                    'FriendlyName'     : friendly_name,
                    'IncludeChain'     : include_chain,
                    'IncludePrivateKey': include_private_key,
                    'KeystorePassword' : keystore_password,
                    'Password'         : password,
                    'RootFirstOrder'   : root_first_order
                }

                class Output(WebSdkOutputModel):
                    certificate_data: str = ApiField(alias='CertificateData')
                    filename: str = ApiField(alias='Filename')
                    format: certificate.CertificateFormat = ApiField(alias='Format')

                return generate_output(response=self._post(data=body), output_cls=Output)

    class _Retry(WebSdkEndpoint):
        def post(self, certificate_dn: str, work_to_do_timeout: int = None):
            body = {
                'CertificateDN'  : certificate_dn,
                'WorkToDoTimeout': work_to_do_timeout
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Revoke(WebSdkEndpoint):
        def post(
            self, certificate_dn: str = None, thumbprint: str = None, reason: int = None, comments: str = None,
            disable: bool = None, work_to_do_timeout: int = None
        ):
            body = {
                'CertificateDN'  : certificate_dn,
                'Thumbprint'     : thumbprint,
                'Reason'         : reason,
                'Comments'       : comments,
                'Disable'        : disable,
                'WorkToDoTimeout': work_to_do_timeout
            }

            class Output(WebSdkOutputModel):
                requested: bool = ApiField(alias='Requested')
                success: bool = ApiField(alias='Success')

            return generate_output(response=self._post(data=body), output_cls=Output)

    class _Validate(WebSdkEndpoint):
        def post(self, certificate_dns: list[str] = None, certificate_guids: list[str] = None):
            body = {
                'CertificateDNs'  : certificate_dns,
                'CertificateGUIDs': certificate_guids
            }

            class Output(WebSdkOutputModel):
                success: bool = ApiField(alias='Success')
                validated_certificate_dns: list[str] = ApiField(default_factory=list, alias='ValidatedCertificateDNs')
                validated_certificate_guids: list[str] = ApiField(
                    default_factory=list,
                    alias='ValidatedCertificateGUIDs'
                )
                warnings: list[str] = ApiField(default_factory=list, alias='Warnings')

            return generate_output(response=self._post(data=body), output_cls=Output)



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    generate_output,
    WebSdkEndpoint,
    WebSdkOutputModel,
)
from pyvenafi.tpp.api.websdk.models import oauth

class OauthOutputBase(WebSdkOutputModel):
    success: bool = ApiField(alias='Success')
    result: bool = ApiField(alias='Result')
    error_description: str = ApiField(alias='error_description')

class _Oauth(WebSdkEndpoint):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/oauth')

        self.GrantRole = self._GrantRole(api_obj=api_obj, url=f'{self._url}/grantrole')
        self.RevokeRole = self._RevokeRole(api_obj=api_obj, url=f'{self._url}/revokerole')
        self.ListRoles = self._ListRoles(api_obj=api_obj, url=f'{self._url}/listroles')
        self.GetRole = self._GetRole(api_obj=api_obj, url=f'{self._url}/getrole')
        self.GetConfiguration = self._GetConfiguration(api_obj=api_obj, url=f'{self._url}/getconfiguration')
        self.SetConfiguration = self._SetConfiguration(api_obj=api_obj, url=f'{self._url}/setconfiguration')
        self.CreateApplication = self._CreateApplication(api_obj=api_obj, url=f'{self._url}/createapplication')
        self.EnumerateApplications = self._Enumerateapplications(
            api_obj=api_obj,
            url=f'{self._url}/enumerateapplications'
        )
        self.GetApplications = self._GetApplications(api_obj=api_obj, url=f'{self._url}/getapplications')
        self.GetApplication = self._GetApplication(api_obj=api_obj, url=f'{self._url}/getapplication')
        self.UpdateApplication = self._UpdateApplication(api_obj=api_obj, url=f'{self._url}/updateapplication')
        self.DeleteApplication = self._DeleteApplication(api_obj=api_obj, url=f'{self._url}/deleteapplication')
        self.GetScopes = self._GetScopes(api_obj=api_obj, url=f'{self._url}/getscopes')
        self.GetGrants = self._GetGrants(api_obj=api_obj, url=f'{self._url}/getgrants')
        self.EnumerateGrants = self._EnumerateGrants(api_obj=api_obj, url=f'{self._url}/enumerategrants')
        self.RevokeGrants = self._RevokeGrants(api_obj=api_obj, url=f'{self._url}/revokegrants')
        self.CreateRule = self._CreateRule(api_obj=api_obj, url=f'{self._url}/createrule')
        self.GetRules = self._GetRules(api_obj=api_obj, url=f'{self._url}/getrules')
        self.EnumerateRules = self._EnumerateRules(api_obj=api_obj, url=f'{self._url}/enumeraterules')
        self.UpdateRule = self._UpdateRule(api_obj=api_obj, url=f'{self._url}/updaterule')
        self.DeleteRule = self._DeleteRule(api_obj=api_obj, url=f'{self._url}/deleterule')
        self.DeleteRules = self._DeleteRules(api_obj=api_obj, url=f'{self._url}/deleterules')
        self.GrantCount = self._GrantCount(api_obj=api_obj, url=f'{self._url}/grantcount')
        self.UserCount = self._UserCount(api_obj=api_obj, url=f'{self._url}/usercount')
        self.CreateJwtMapping = self._CreateJwtMapping(api_obj=api_obj, url=f'{self._url}/createjwtmapping')
        self.EnumerateJwtMappings = self._EnumerateJwtMappings(api_obj=api_obj, url=f'{self._url}/enumeratejwtmappings')
        self.GetJwtMappings = self._GetJwtMappings(api_obj=api_obj, url=f'{self._url}/getjwtmappings')
        self.GetJwtMapping = self._GetJwtMapping(api_obj=api_obj, url=f'{self._url}/getjwtmapping')
        self.UpdateJwtMapping = self._UpdateJwtMapping(api_obj=api_obj, url=f'{self._url}/updatejwtmapping')
        self.DeleteJwtMapping = self._DeleteJwtMapping(api_obj=api_obj, url=f'{self._url}/deletejwtmapping')

    class _GrantRole(WebSdkEndpoint):
        def post(self, grantee_prefixed_universal: str = None, role: int = None, application_id: str = None):
            body = {
                'GranteePrefixedUniversal': grantee_prefixed_universal,
                'Role'                    : role,
                'ApplicationId'           : application_id,
            }

            class Output(OauthOutputBase):
                success: bool = ApiField(alias='Success')
                result: bool = ApiField(alias='Result')
                error: bool = ApiField(alias='Error')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RevokeRole(WebSdkEndpoint):
        def post(self, grantee_prefixed_universal: str = None, role: int = None):
            body = {
                "GranteePrefixedUniversal": grantee_prefixed_universal,
                "Role"                    : role
            }

            class Output(OauthOutputBase):
                success: bool = ApiField(alias='Success')
                result: bool = ApiField(alias='Result')
                error: bool = ApiField(alias='Error')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _ListRoles(WebSdkEndpoint):
        def post(self, grantee_prefixed_universal: str = None):
            body = {
                "GranteePrefixedUniversal": grantee_prefixed_universal
            }

            class Output(OauthOutputBase):
                roles: list[oauth.Role] = ApiField(alias='Roles')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetRole(WebSdkEndpoint):
        def post(self):
            class Output(OauthOutputBase):
                role: oauth.Role = ApiField(alias='Role')

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _GetConfiguration(WebSdkEndpoint):
        def post(self):
            class Output(OauthOutputBase):
                configuration: oauth.Configuration = ApiField(alias='Configuration')

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _SetConfiguration(WebSdkEndpoint):
        def post(self, configuration: oauth.Configuration | dict[str, any]):
            if isinstance(configuration, oauth.Configuration):
                configuration = configuration.dict(by_alias=True)
            body = {
                "Configuration": configuration
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CreateApplication(WebSdkEndpoint):
        def post(
            self, application_id: str = None, scope: str = None, maximum_scope: str = None,
            name: str = None, vendor: str = None, description: str = None, url: str = None,
            access_validity: int = None, grant_validity: int = None, renewable: bool = None,
        ):
            body = {
                "ApplicationId" : application_id,
                "Scope"         : scope,
                "MaximumScope"  : maximum_scope,
                "Name"          : name,
                "Vendor"        : vendor,
                "Description"   : description,
                "Url"           : url,
                "AccessValidity": access_validity,
                "GrantValidity" : grant_validity,
                "Renewable"     : renewable,
            }

            class Output(OauthOutputBase):
                application: oauth.Application = ApiField(alias='Application')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _Enumerateapplications(WebSdkEndpoint):
        def post(self, start: int = None, count: int = None, order_by: int = 0, descending: bool = True):
            body = {
                "Start"     : start,
                "Count"     : count,
                "OrderBy"   : order_by,
                "Descending": descending,
            }

            class Output(OauthOutputBase):
                applications: list[oauth.Application] = ApiField(alias='Applications')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetApplications(WebSdkEndpoint):
        def post(self, filter: str = None, application_ids: list[str] = None):
            body = {
                "Filter"        : filter,
                "ApplicationIds": application_ids,
            }

            class Output(OauthOutputBase):
                applications: list[oauth.Application] = ApiField(alias='Applications')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetApplication(WebSdkEndpoint):
        def post(self, application_id: int):
            body = {
                'ApplicationId': application_id,
            }

            class Output(OauthOutputBase):
                application: oauth.Application = ApiField(alias='Application')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UpdateApplication(WebSdkEndpoint):
        def post(self, application: oauth.Application | dict):
            if isinstance(application, oauth.Application):
                application = application.dict(by_alias=True)

            body = {
                "Application": application,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _DeleteApplication(WebSdkEndpoint):
        def post(self, application_id: int):
            body = {
                "ApplicationId": application_id,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetScopes(WebSdkEndpoint):
        def post(self):
            class Output(OauthOutputBase):
                scopes: list[oauth.Scope]

            return generate_output(output_cls=Output, response=self._post(data={}))

    class _GetGrants(WebSdkEndpoint):
        def post(
            self,
            grantee_prefixed_universal: str = None,
            application_id: str = None,
            start: int = None,
            count: int = None,
            order_by: int = None,
            descending: bool = None,
            resolve_identities: bool = None,
        ):
            body = {
                "GranteePrefixedUniversal": grantee_prefixed_universal,
                "ApplicationId"           : application_id,
                "Start"                   : start,
                "Count"                   : count,
                "OrderBy"                 : order_by,
                "Descending"              : descending,
                "ResolveIdentities"       : resolve_identities,
            }

            class Output(OauthOutputBase):
                grants: list[oauth.Grant] = ApiField(alias='Grants')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _EnumerateGrants(WebSdkEndpoint):
        def post(
            self,
            application_id: str = None,
            start: str = None,
            count: int = None,
            order_by: int = None,
            descending: bool = None,
            resolve_identities: bool = None,
        ):
            body = {
                "ApplicationId"    : application_id,
                "Start"            : start,
                "Count"            : count,
                "OrderBy"          : order_by,
                "Descending"       : descending,
                "ResolveIdentities": resolve_identities,
            }

            class Output(OauthOutputBase):
                grants: list[oauth.Grant] = ApiField(alias='Grants')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _RevokeGrants(WebSdkEndpoint):
        def post(
            self,
            application_id: str = None,
            grantee_prefixed_universal: str = None,
        ):
            body = {
                "ApplicationId"           : application_id,
                "GranteePrefixedUniversal": grantee_prefixed_universal,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CreateRule(WebSdkEndpoint):
        def post(
            self,
            application_id: str = None,
            trustee_prefixed_universal: str = None,
            maximum_scope: str = None,
            description: str = None,
            access_validity: int = None,
            grant_validity: int = None,
            renewable: bool = None,
        ):
            body = {
                "ApplicationId"           : application_id,
                "TrusteePrefixedUniversal": trustee_prefixed_universal,
                "MaximumScope"            : maximum_scope,
                "Description"             : description,
                "AccessValidity"          : access_validity,
                "GrantValidity"           : grant_validity,
                "Renewable"               : renewable,
            }

            class Output(OauthOutputBase):
                rule: oauth.Rule = ApiField(alias='Rule')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetRules(WebSdkEndpoint):
        def post(
            self,
            application_id: str = None,
            trustee_prefixed_universal: str = None,
            trustee_prefixed_universals: list[str] = None,
            maximum_scope: str = None,
            renewable: bool = None,
            no_expire: bool = None,
            resolve_identities: bool = None,
        ):
            body = {
                "ApplicationId"            : application_id,
                "TrusteePrefixedUniversal" : trustee_prefixed_universal,
                "TrusteePrefixedUniversals": trustee_prefixed_universals,
                "MaximumScope"             : maximum_scope,
                "Renewable"                : renewable,
                "NoExpire"                 : no_expire,
                "ResolveIdentities"        : resolve_identities,
            }

            class Output(OauthOutputBase):
                rules: list[oauth.Rule] = ApiField(alias='Rules')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _EnumerateRules(WebSdkEndpoint):
        def post(
            self,
            application_id: str = None,
            start: int = None,
            count: int = None,
            order_by: int = None,
            descending: bool = None,
            resolve_identities: bool = None,
        ):
            body = {
                "ApplicationId"    : application_id,
                "Start"            : start,
                "Count"            : count,
                "OrderBy"          : order_by,
                "Descending"       : descending,
                "ResolveIdentities": resolve_identities,
            }

            class Output(OauthOutputBase):
                rules: list[oauth.Rule] = ApiField(alias='Rules')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UpdateRule(WebSdkEndpoint):
        def post(self, rule: oauth.Rule | dict[str, any]):
            if isinstance(rule, oauth.Rule):
                rule = rule.dict(by_alias=True)
            body = {
                "Rule": rule,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _DeleteRule(WebSdkEndpoint):
        def post(
            self,
            trustee_prefixed_universal: str,
            application_id: str,
        ):
            body = {
                "TrusteePrefixedUniversal": trustee_prefixed_universal,
                "ApplicationId"           : application_id,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _DeleteRules(WebSdkEndpoint):
        def post(
            self,
            trustee_prefixed_universal: str,
            application_id: str,
        ):
            body = {
                "TrusteePrefixedUniversal": trustee_prefixed_universal,
                "ApplicationId"           : application_id,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GrantCount(WebSdkEndpoint):
        def post(self, application_id: str):
            body = {
                "ApplicationId": application_id,
            }

            class Output(OauthOutputBase):
                count: int = ApiField(alias='Count')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UserCount(WebSdkEndpoint):
        def post(self, application_id: str):
            body = {
                "ApplicationId": application_id,
            }

            class Output(OauthOutputBase):
                count: int = ApiField(alias='Count')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _CreateJwtMapping(WebSdkEndpoint):
        def post(
            self,
            name=None,
            issuer_uri=None,
            purpose_field=None,
            purpose_match=None,
            id_field=None,
            id_match=None,
            grantee_prefixed_universal=None,
        ):
            body = {
                "Name"                    : name,
                "IssuerUri"               : issuer_uri,
                "PurposeField"            : purpose_field,
                "PurposeMatch"            : purpose_match,
                "IdField"                 : id_field,
                "IdMatch"                 : id_match,
                "GranteePrefixedUniversal": grantee_prefixed_universal,
            }

            class Output(OauthOutputBase):
                jwt_mapping: oauth.JwtMapping = ApiField(alias='JwtMapping')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _EnumerateJwtMappings(WebSdkEndpoint):
        def post(
            self,
            start: int = None,
            count: int = None,
            order_by: int = None,
            descending: bool = None,
            resolve_identities: bool = None,
        ):
            body = {
                "Start"            : start,
                "Count"            : count,
                "OrderBy"          : order_by,
                "Descending"       : descending,
                "ResolveIdentities": resolve_identities,
            }

            class Output(OauthOutputBase):
                jwt_mappings: list[oauth.JwtMapping] = ApiField(alias='JwtMappings')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetJwtMappings(WebSdkEndpoint):
        def post(
            self,
            filter: str = None,
            issuer_uris: list[str] = None,
            resolve_identities: bool = None
        ):
            body = {
                "Filter"           : filter,
                "IssuerUris"       : issuer_uris,
                "ResolveIdentities": resolve_identities,
            }

            class Output(OauthOutputBase):
                jwt_mappings: list[oauth.JwtMapping] = ApiField(alias='JwtMappings')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _GetJwtMapping(WebSdkEndpoint):
        def post(
            self,
            name: str,
            resolve_identities: bool = None,
        ):
            body = {
                "Name"             : name,
                "ResolveIdentities": resolve_identities,
            }

            class Output(OauthOutputBase):
                jwt_mapping: oauth.JwtMapping = ApiField(alias='JwtMapping')

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _UpdateJwtMapping(WebSdkEndpoint):
        def post(self, jwt_mapping: oauth.JwtMapping):
            if isinstance(jwt_mapping, oauth.JwtMapping):
                jwt_mapping = jwt_mapping.dict(by_alias=True)
            body = {
                'JwtMapping': jwt_mapping,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))

    class _DeleteJwtMapping(WebSdkEndpoint):
        def post(self, name: str):
            body = {
                "Name": name,
            }

            class Output(OauthOutputBase):
                pass

            return generate_output(output_cls=Output, response=self._post(data=body))



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.http_ca_base import HTTPCABaseAttributes

class OpenTrustPKICAAttributes(HTTPCABaseAttributes, metaclass=IterableMeta):
    __config_class__ = "OpenTrust PKI CA"
    connector_type = Attribute('Connector Type', min_version='21.4')
    fields = Attribute('Fields', min_version='21.4')
    retrieval_period = Attribute('Retrieval Period', min_version='21.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import IterableMeta
from pyvenafi.tpp.attributes.x509_certificate import X509CertificateAttributes

class X509DeviceCertificateAttributes(X509CertificateAttributes, metaclass=IterableMeta):
    __config_class__ = "X509 Device Certificate"



from __future__ import annotations

from datetime import datetime
from typing import (
    Generic,
    Literal,
    TypeVar,
)

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)
from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes

T = TypeVar('T')

EnvironmentType = Literal[
    'Code Signing Apple Environment',
    'Code Signing Certificate Environment',
    'Code Signing CSP Environment',
    'Code Signing DotNet Environment',
    'Code Signing Key Pair Environment',
    'Code Signing GPG Environment'
]

TemplateType = Literal[
    'Code Signing Apple Environment Template',
    'Code Signing Certificate Environment Template',
    'Code Signing CSP Environment Template',
    'Code Signing DotNet Environment Template',
    'Code Signing Key Pair Environment Template',
    'Code Signing GPG Environment Template',
]

# region Models
class ResultCode(ObjectModel):
    code: int = ApiField()

    @property
    def codesign_result(self):
        return ResultCodes.CodeSign.get(self.code, 'Unknown')

class Items(ObjectModel, Generic[T]):
    dirty: bool = ApiField(alias='Dirty')
    items: list[T] = ApiField(alias='Items', default_factory=list)

class InfoValue(ObjectModel, Generic[T]):
    info: int = ApiField(alias='Info')
    value: Items[T] = ApiField(alias='Value')

class CustomFieldAttributes(ObjectModel):
    field_name: str = ApiField(alias='FieldName')
    values: list[str] = ApiField(alias='Values', default_factory=list)

class EnvironmentTemplateDetails(ObjectModel, Generic[T]):
    dirty: bool = ApiField(alias='Dirty', default=None)
    info: int = ApiField(alias='Info')
    template_values: InfoValue[T] = ApiField(alias='TemplateValues')
    value: T = ApiField(alias='Value')

class RightsKeyValue(ObjectModel):
    key: str = ApiField(alias='Key')
    value: int = ApiField(alias='Value')

class SignApplicationCollection(ObjectModel):
    description: str = ApiField(alias='Description')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    hash: str = ApiField(alias='Hash')
    id: int = ApiField(alias='Id')
    location: str = ApiField(alias='Location')
    permitted_argument_pattern: str = ApiField(alias='PermittedArgumentPattern')
    signatory_issuer: str = ApiField(alias='SignatoryIssuer')
    signatory_subject: str = ApiField(alias='SignatorySubject')
    size: int = ApiField(alias='Size')
    version: str = ApiField(alias='Version')

class AppleTemplate(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    certificate_authority_dn: InfoValue[str] = ApiField(alias='CertificateAuthorityDN')
    certificate_subject: InfoValue[str] = ApiField(alias='CertificateSubject')
    city: InfoValue[str] = ApiField(alias='City')
    country: InfoValue[str] = ApiField(alias='Country')
    cn_pattern: InfoValue[str] = ApiField(alias='CNPattern')
    description: str = ApiField(alias='Description')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
    organization: InfoValue[str] = ApiField(alias='Organization')
    organizational_unit: InfoValue[str] = ApiField(alias='OrganizationalUnit')
    per_user: bool = ApiField(alias='PerUser')
    read_only: bool = ApiField(alias='ReadOnly')
    state: InfoValue[str] = ApiField(alias='State')
    target_policy_dn: str = ApiField(alias='TargetPolicyDN')
    type: TemplateType = ApiField(alias='Type')
    visible_to: Items[str] = ApiField(alias='VisibleTo')

class AppleEnvironment(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    apple_template: AppleTemplate = ApiField(alias='AppleTemplate')
    custom_field_attributes: Items[CustomFieldAttributes] = ApiField(alias='CustomFieldAttributes')
    dirty: bool = ApiField(alias='Dirty')
    disabled: bool = ApiField(alias='Disabled')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    per_user: bool = ApiField(alias='PerUser')
    template_dn: str = ApiField(alias='TemplateDN')
    type: EnvironmentType = ApiField(alias='Type')

class CertificateTemplate(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    certificate_authority_dn: InfoValue[str] = ApiField(alias='CertificateAuthorityDN')
    certificate_subject: str = ApiField(alias='CertificateSubject')
    certificate_stage: str = ApiField(alias='CertificateStage')
    certificate_status_text: str = ApiField(alias='CertificateStatusText')
    city: InfoValue[str] = ApiField(alias='City')
    country: InfoValue[str] = ApiField(alias='Country')
    custom_field_attributes: Items[CustomFieldAttributes] = ApiField(alias='CustomFieldAttributes')
    description: str = ApiField(alias='Description')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
    organization: InfoValue[str] = ApiField(alias='Organization')
    organizational_unit: InfoValue[str] = ApiField(alias='OrganizationalUnit')
    per_user: bool = ApiField(alias='PerUser')
    read_only: bool = ApiField(alias='ReadOnly')
    san_email: InfoValue[str] = ApiField(alias='SANEmail')
    state: InfoValue[str] = ApiField(alias='State')
    target_policy_dn: str = ApiField(alias='TargetPolicyDN')
    type: TemplateType = ApiField(alias='Type')
    visible_to: Items[str] = ApiField(alias='VisibleTo')

class CertificateEnvironment(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    ca_specific_attributes: Items[str] = ApiField(alias='CASpecificAttributes')
    certificate_authority_dn: EnvironmentTemplateDetails[str] = ApiField(alias='CertificateAuthorityDN')
    certificate_dn: str = ApiField(alias='CertificateDN')
    certificate_status: int = ApiField(alias='CertificateStatus')
    certificate_status_text: int = ApiField(alias='CertificateStatusText')
    certificate_subject: EnvironmentTemplateDetails[str] = ApiField(alias='CertificateSubject')
    certificate_template: CertificateTemplate = ApiField(alias='CertificateTemplate')
    city: EnvironmentTemplateDetails[str] = ApiField(alias='City')
    country: EnvironmentTemplateDetails[str] = ApiField(alias='Country')
    custom_field_attributes: CustomFieldAttributes = ApiField(alias='CustomFieldAttributes')
    disabled: bool = ApiField(alias='Disabled')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    ip_address_restriction: Items[str] = ApiField(alias='IPAddressRestriction')
    key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='KeyAlgorithm')
    key_storage_location: EnvironmentTemplateDetails[str] = ApiField(alias='KeyStorageLocation')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    organization: EnvironmentTemplateDetails[str] = ApiField(alias='Organization')
    organizational_unit: EnvironmentTemplateDetails[str] = ApiField(alias='OrganizationalUnit')
    per_user: bool = ApiField(alias='PerUser')
    san_email: EnvironmentTemplateDetails[str] = ApiField(alias='SANEmail')
    state: EnvironmentTemplateDetails[str] = ApiField(alias='State')
    status: int = ApiField(alias='Status')
    target_store: EnvironmentTemplateDetails[str] = ApiField(alias='TargetStore')
    template_dn: str = ApiField(alias='TemplateDN')
    type: EnvironmentType = ApiField(alias='Type')

class CertificateEnvironment_Pre22_4(CertificateEnvironment):
    class Config:
        fields = {
            'country': {
                'alias': 'country'
            }
        }

class CSPTemplate(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    description: str = ApiField(alias='Description')
    dirty: bool = ApiField(alias='Dirty')
    dn: str = ApiField(alias='Dn')
    encryption_key_algorithm: InfoValue[str] = ApiField(alias='EncryptionKeyAlgorithm')
    expiration: InfoValue[str] = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_container_dn: str = ApiField(alias='KeyContainerDN')
    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
    max_uses: InfoValue[str] = ApiField(alias='MaxUses')
    object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
    per_user: bool = ApiField(alias='PerUser')
    signing_key_algorithm: InfoValue[str] = ApiField(alias='SigningKeyAlgorithm')
    type: TemplateType = ApiField(alias='Type')
    visible_to: Items[str] = ApiField(alias='VisibleTo')

class CSPEnvironment(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    csp_template: CSPTemplate = ApiField(alias='CspTemplate')
    disabled: bool = ApiField(alias='Disabled')
    dn: str = ApiField(alias='Dn')
    encryption_key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='EncryptionKeyAlgorithm')
    encryption_key_dn: str = ApiField(alias='EncryptionKeyDN')
    expiration: int = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
    key_storage_location: EnvironmentTemplateDetails[str] = ApiField(alias='KeyStorageLocation')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    max_uses: int = ApiField(alias='MaxUses')
    per_user: bool = ApiField(alias='PerUser')
    signing_key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='SigningKeyAlgorithm')
    signing_key_dn: str = ApiField(alias='SigningKeyDN')
    template_dn: str = ApiField(alias='TemplateDN')

class DotNetTemplate(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    description: str = ApiField(alias='Description')
    disabled: bool = ApiField(alias='Disabled')
    dn: str = ApiField(alias='Dn')
    expiration: InfoValue[str] = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
    key_container_dn: InfoValue[str] = ApiField(alias='KeyContainerDN')
    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    max_uses: InfoValue[str] = ApiField(alias='MaxUses')
    object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
    per_user: bool = ApiField(alias='PerUser')
    type: TemplateType = ApiField(alias='Type')
    visible_to: Items[str] = ApiField(alias='VisibleTo')

class DotNetEnvironment(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    dot_net_template: DotNetTemplate = ApiField(alias='DotNetTemplate')
    disabled: bool = ApiField(alias='Disabled')
    dn: str = ApiField(alias='Dn')
    expiration: int = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
    key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='KeyAlgorithm')
    key_dn: str = ApiField(alias='KeyDN')
    key_storage_location: EnvironmentTemplateDetails[str] = ApiField(alias='KeyStorageLocation')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    max_uses: int = ApiField(alias='MaxUses')
    per_user: bool = ApiField(alias='PerUser')
    template_dn: str = ApiField(alias='TemplateDN')
    type: TemplateType = ApiField(alias='Type')

class GPGTemplate(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    authentication_key_algorithm: InfoValue[str] = ApiField(alias='AuthenticationKeyAlgorithm')
    description: str = ApiField(alias='Description')
    dn: str = ApiField(alias='Dn')
    email: InfoValue[str] = ApiField(alias='Email')
    encryption_key_algorithm: InfoValue[str] = ApiField(alias='EncryptionKeyAlgorithm')
    expiration: InfoValue[str] = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_container_dn: str = ApiField(alias='KeyContainerDN')
    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    max_uses: InfoValue[str] = ApiField(alias='MaxUses')
    object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
    per_user: bool = ApiField(alias='PerUser')
    read_only: bool = ApiField(alias='ReadOnly')
    real_name: InfoValue[str] = ApiField(alias='RealName')
    signing_key_algorithm: InfoValue[str] = ApiField(alias='SigningKeyAlgorithm')
    type: TemplateType = ApiField(alias='Type')
    visible_to: Items[str] = ApiField(alias='VisibleTo')

class GPGEnvironment(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    authentication_key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='AuthenticationKeyAlgorithm')
    custom_field_attributes: Items[CustomFieldAttributes] = ApiField(alias='CustomFieldAttributes')
    dirty: bool = ApiField(alias='Dirty')
    dn: str = ApiField(alias='Dn')
    email: list[str] = ApiField(alias='Email', default_factory=list)
    encryption_key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='EncryptionKeyAlgorithm')
    expiration: int = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    gpg_template: GPGTemplate = ApiField(alias='GpgTemplate')
    id: int = ApiField(alias='Id')
    ip_address_restriction: Items[str] = ApiField(alias='IpAddressRestriction')
    key_storage_location: EnvironmentTemplateDetails[str] = ApiField(alias='KeyStorageLocation')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    max_uses: int = ApiField(alias='MaxUses')
    per_user: bool = ApiField(alias='PerUser')
    real_name: EnvironmentTemplateDetails[str] = ApiField(alias='RealName')
    read_only: bool = ApiField(alias='ReadOnly')
    signing_key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='SigningKeyAlgorithm')
    status: int = ApiField(alias='Status')
    template_dn: str = ApiField(alias='TemplateDN')
    type: EnvironmentType = ApiField(alias='Type')

class KeyPairTemplate(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    description: str = ApiField(alias='Description')
    dirty: bool = ApiField(alias='Dirty')
    dn: str = ApiField(alias='Dn')
    expiration: InfoValue[str] = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_algorithm: InfoValue[str] = ApiField(alias='KeyAlgorithm')
    key_container_dn: str = ApiField(alias='KeyContainerDN')
    key_storage_location: InfoValue[str] = ApiField(alias='KeyStorageLocation')
    max_uses: int = ApiField(alias='MaxUses')
    object_naming_pattern: str = ApiField(alias='ObjectNamingPattern')
    per_user: bool = ApiField(alias='PerUser')
    read_only: bool = ApiField(alias='ReadOnly')
    type: TemplateType = ApiField(alias='Type')
    validity_period: int = ApiField(alias='ValidityPeriod')
    visible_to: Items[str] = ApiField(alias='VisibleTo')

class KeyPairEnvironment(ObjectModel):
    allow_user_key_import: bool = ApiField(alias='AllowUserKeyImport')
    custom_field_attributes: Items[CustomFieldAttributes] = ApiField(alias='CustomFieldAttributes')
    dn: str = ApiField(alias='Dn')
    disabled: bool = ApiField(alias='Disabled')
    expiration: int = ApiField(alias='Expiration')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    ip_address_restriction: Items[str] = ApiField(alias='IPAddressRestriction')
    key_algorithm: EnvironmentTemplateDetails[str] = ApiField(alias='KeyAlgorithm')
    key_dn: str = ApiField(alias='KeyDN')
    key_pair_template: KeyPairTemplate = ApiField(alias='KeyPairTemplate')
    key_storage_location: EnvironmentTemplateDetails[str] = ApiField(alias='KeyStorageLocation')
    key_time_constraints: Items[str] = ApiField(alias='KeyTimeConstraints')
    key_usage: EnvironmentTemplateDetails[str] = ApiField(alias='KeyUsage')
    key_use_flow_dn: str = ApiField(alias='KeyUseFlowDN')
    status: int = ApiField(alias='Status')
    template_dn: str = ApiField(alias='TemplateDN')
    type: EnvironmentType = ApiField(alias='Type')

class Application(ObjectModel):
    description: str = ApiField(alias='Description')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    hash: str = ApiField(alias='Hash')
    id: int = ApiField(alias='Id')
    location: str = ApiField(alias='Location')
    permitted_argument_pattern: str = ApiField(alias='PermittedArgumentPattern')
    signatory_issuer: str = ApiField(alias='SignatoryIssuer')
    signatory_subject: str = ApiField(alias='SignatorySubject')
    size: int = ApiField(alias='Size')
    version: str = ApiField(alias='Version')

class ApplicationCollection(ObjectModel):
    application_dns: Items[str] = ApiField(alias='ApplicationDNs')
    dn: str = ApiField(alias='Dn')
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')

class GlobalConfiguration(ObjectModel):
    approved_key_storage_locations: Items[str] = ApiField(alias='ApprovedKeyStorageLocations')
    available_key_storage_locations: Items[str] = ApiField(alias='AvailableKeyStorageLocations')
    default_ca_container: str = ApiField(alias='DefaultCAContainer')
    default_certificate_container: str = ApiField(alias='DefaultCertificateContainer')
    default_credential_container: str = ApiField(alias='DefaultCredentialContainer')
    key_use_timeout: int = ApiField(alias='KeyUseTimeout')
    project_description_tooltip: str = ApiField(alias='ProjectDescriptionTooltip')
    request_in_progress_message: str = ApiField(alias='RequestInProgressMessage')

class Project(ObjectModel):
    application_dns: Items[str] = ApiField(alias='ApplicationDNs')
    applications: list[Application] = ApiField(alias='Applications', default_factory=list)
    auditors: Items[str] = ApiField(alias='Auditors')
    certificate_environments: list[CertificateEnvironment] = ApiField(
        alias='CertificateEnvironments',
        default_factory=list
    )
    collections: list[ApplicationCollection] = ApiField(alias='Collections', default_factory=list)
    created_on: datetime = ApiField(alias='CreatedOn')
    csp_environments: list[CSPEnvironment] = ApiField(alias='CSPEnvironments', default_factory=list)
    custom_field_attributes: Items[CustomFieldAttributes] = ApiField(
        alias='CustomFieldAttributes',
        default_factory=list
    )
    description: str = ApiField(alias='Description')
    dn: str = ApiField(alias='Dn')
    dot_net_environments: list[DotNetEnvironment] = ApiField(alias='DotNetEnvironments', default_factory=list)
    gpg_environments: list[GPGEnvironment] = ApiField(alias='GPGEnvironments', default_factory=list)
    guid: str = ApiField(alias='Guid')
    id: int = ApiField(alias='Id')
    key_use_approvers: Items[str] = ApiField(alias='KeyUseApprovers')
    key_users: Items[str] = ApiField(alias='KeyUsers')
    owners: Items[str] = ApiField(alias='Owners')
    status: int = ApiField(alias='Status')

class Rights(ObjectModel):
    value: int = ApiField()

    @property
    def none(self) -> int:
        return self.value == 0

    @property
    def admin(self) -> int:
        return self.value & 1 == 1

    @property
    def use(self) -> int:
        return self.value & 2 == 2

    @property
    def audit(self) -> int:
        return self.value & 4 == 4

    @property
    def owner(self) -> int:
        return self.value & 8 == 8

    @property
    def project_approval(self) -> int:
        return self.value & 16 == 16

    @property
    def application_admin(self) -> int:
        return self.value & 32 == 32

    @property
    def approve_use(self) -> int:
        return self.value & 64 == 64

class ArchiveFilter(ObjectModel):
    application_hash: str = ApiField(alias='ApplicationHash')
    artifact: str = ApiField(alias='Artifact')
    authenticated_user: str = ApiField(alias='AuthenticatedUser')
    client_interface: int = ApiField(alias='ClientInterface')
    client_library_name: str = ApiField(alias='ClientLibraryName')
    client_library_version: str = ApiField(alias='ClientLibraryVersion')
    client_mechanism: str = ApiField(alias='ClientMechanism')
    command: str = ApiField(alias='Command')
    environment_dn: str = ApiField(alias='EnvironmentDN')
    event_id: int = ApiField(alias='EventId')
    executable: str = ApiField(alias='Executable')
    executable_location: str = ApiField(alias='ExecutableLocation')
    flow_dn: str = ApiField(alias='FlowDN')
    grouping: int = ApiField(alias='Grouping')
    ip_address: str = ApiField(alias='IPAddress')
    key_type: str = ApiField(alias='KeyType')
    key_vault_id: int = ApiField(alias='KeyVaultId')
    machine: str = ApiField(alias='Machine')
    mechanism: int = ApiField(alias='Mechanism')
    project_dn: str = ApiField(alias='ProjectDN')
    project_dns: str = ApiField(alias='ProjectDNs')
    remote_account: str = ApiField(alias='RemoteAccount')
    timestamp_after: datetime = ApiField(alias='TimestampAfter')
    timestamp_before: datetime = ApiField(alias='TimestampBefore')

class ArchiveResults(ObjectModel):
    application_hash: str = ApiField(alias='ApplicationHash')
    archive_entry_id: str = ApiField(alias='ArchiveEntryId')
    artifact: str = ApiField(alias='Artifact')
    authenticated_user: str = ApiField(alias='AuthenticatedUser')
    client_library_location: str = ApiField(alias='ClientLibraryLocation')
    client_library_name: str = ApiField(alias='ClientLibraryName')
    client_library_version: str = ApiField(alias='ClientLibraryVersion')
    client_mechanism: str = ApiField(alias='ClientMechanism')
    command: str = ApiField(alias='Command')
    environment: str = ApiField(alias='Environment')
    event_id: int = ApiField(alias='EventId')
    executable: str = ApiField(alias='Executable')
    executable_location: str = ApiField(alias='ExecutableLocation')
    flow: str = ApiField(alias='Flow')
    grouping: int = ApiField(alias='Grouping')
    ip_address: str = ApiField(alias='IPAddress')
    key_type: str = ApiField(alias='KeyType')
    key_vault_id: int = ApiField(alias='KeyVaultId')
    machine: str = ApiField(alias='Machine')
    mechanism: int = ApiField(alias='Mechanism')
    platform: str = ApiField(alias='Platform')
    project: str = ApiField(alias='Project')
    remote_account: str = ApiField(alias='RemoteAccount')
    timestamp: datetime = ApiField(alias='Timestamp')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class ACMEAccountAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "ACME Account"
    account_key_hash = Attribute('Account Key Hash')
    certificate_location_dn = Attribute('Certificate Location DN')
    internet_email_address = Attribute('Internet EMail Address')
    json_web_key = Attribute('JSON Web Key')
    public_key_vault_id = Attribute('Public Key Vault Id')
    status = Attribute('Status')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.code_signing_environment_template_base import CodeSigningEnvironmentTemplateBaseAttributes

class CodeSigningAppleEnvironmentTemplateAttributes(
    CodeSigningEnvironmentTemplateBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "Code Signing Apple Environment Template"
    cn_pattern = Attribute('CN Pattern', min_version='21.4')
    certificate_authority = Attribute('Certificate Authority', min_version='22.1')
    certificate_container_dn = Attribute('Certificate Container DN', min_version='21.4')
    key_container_dn = Attribute('Key Container DN', min_version='21.4')
    key_storage_location = Attribute('Key Storage Location', min_version='21.4')
    max_uses = Attribute('Max Uses', min_version='21.4')
    validity_period = Attribute('Validity Period', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.client_work_base import ClientWorkBaseAttributes

class ClientAgentConfigurationWorkAttributes(ClientWorkBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Client Agent Configuration Work"
    agent_cert_trust_bundle = Attribute('Agent Cert Trust Bundle')
    days_of_month = Attribute('Days Of Month')
    days_of_week = Attribute('Days Of Week')
    event_row_count = Attribute('Event Row Count')
    interval = Attribute('Interval')
    log_communications = Attribute('Log Communications')
    log_facility = Attribute('Log Facility')
    log_threshold = Attribute('Log Threshold')
    minimum_free_space = Attribute('Minimum Free Space')
    proxy_credential = Attribute('Proxy Credential')
    proxy_host = Attribute('Proxy Host')
    schedule_type = Attribute('Schedule Type')
    start_time = Attribute('Start Time')
    tls_crl_verify = Attribute('TLS CRL Verify')
    web_service_url = Attribute('Web Service URL')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.credential_driver_base import CredentialDriverBaseAttributes

class CyberArkCredentialsDriverAttributes(CredentialDriverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "CyberArk Credentials Driver"
    ccp_certificate_credential = Attribute('CCP Certificate Credential', min_version='22.4')
    central_credential_provider_web_service_url = Attribute(
        'Central Credential Provider Web Service URL',
        min_version='21.4'
    )
    password_retrieval_method = Attribute('Password Retrieval Method', min_version='21.4')
    scim_server_url = Attribute('SCIM Server URL', min_version='21.4')
    scim_server_user = Attribute('SCIM Server User', min_version='21.4')
    use_proxy = Attribute('Use Proxy', min_version='21.4')
    user_authentication_method = Attribute('User Authentication Method', min_version='22.4')
    web_service_url = Attribute('Web Service URL', min_version='21.4')
    web_service_user = Attribute('Web Service User', min_version='21.4')
    web_services_authentication_method = Attribute('Web Services Authentication Method', min_version='22.2')
    windows_credential_provider_version = Attribute('Windows Credential Provider Version', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    Attribute,
    IterableMeta,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class BrocadeAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Brocade"
    all_cipher_suites = Attribute('All Cipher Suites')
    certificate_file = Attribute('Certificate File')
    chain_cert = Attribute('Chain Cert')
    file_validation_disabled = Attribute('File Validation Disabled')
    network_validation_disabled = Attribute('Network Validation Disabled')
    private_key_file = Attribute('Private Key File')
    privileged_mode_credential = Attribute('Privileged Mode Credential')
    ssl_profile_name = Attribute('SSL Profile Name')
    selected_cipher_suites = Attribute('Selected Cipher Suites')
    session_cache = Attribute('Session Cache')
    temp_certificate_label = Attribute('Temp Certificate Label')
    virtual_server_name = Attribute('Virtual Server Name')



from __future__ import annotations
from pyvenafi.cloud.api.api_base import (
    ApiField,
    ObjectModel,
)
from typing import (
    Any,
    List,
    Literal,
    Union,
)
from uuid import UUID

AnyValue = Any

class AwsCloudProviderInformation(ObjectModel):
    accountIds: List[str] = ApiField(alias='accountIds', default_factory=list)
    regions: List[Literal[
        'af-south-1', 'ap-east-1', 'ap-northeast-1', 'ap-northeast-2', 'ap-northeast-3', 'ap-south-1', 'ap-south-2', 'ap-southeast-1', 'ap-southeast-2', 'ap-southeast-3', 'ap-southeast-4', 'ca-central-1', 'eu-central-1', 'eu-central-2',
        'eu-north-1', 'eu-south-1', 'eu-south-2', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'me-central-1', 'me-south-1', 'sa-east-1', 'us-east-1', 'us-east-2', 'us-gov-east-1', 'us-gov-west-1', 'us-west-1', 'us-west-2']] = ApiField(
        alias='regions',
        default_factory=list
    )

class AzureCloudProviderInformation(ObjectModel):
    subscriptionIds: List[UUID] = ApiField(alias='subscriptionIds', default_factory=list)

class ClientAuthenticationInformation(ObjectModel):
    type: str = ApiField(alias='type')

class ClientAuthenticationOpenApi(ObjectModel):
    type: str = ApiField(alias='type')

class ClientAuthenticationRequestOpenApi(ObjectModel):
    clientAuthentication: Union[JwtJwksAuthenticationOpenApi, JwtOidcAuthenticationOpenApi] = ApiField(
        alias='clientAuthentication'
    )

class CloudProvidersInformation(ObjectModel):
    aws: AwsCloudProviderInformation = ApiField(alias='aws')
    azure: AzureCloudProviderInformation = ApiField(alias='azure')
    google: GoogleCloudProviderInformation = ApiField(alias='google')

class ConfigurationCreateRequest(ObjectModel):
    clientAuthentication: ClientAuthenticationInformation = ApiField(alias='clientAuthentication')
    cloudProviders: CloudProvidersInformation = ApiField(alias='cloudProviders')
    name: str = ApiField(alias='name')
    policyIds: UUID = ApiField(alias='policyIds')
    serviceAccountIds: List[UUID] = ApiField(alias='serviceAccountIds', default_factory=list)
    subCaProviderId: UUID = ApiField(alias='subCaProviderId')

class ConfigurationDeleteResponse(ObjectModel):
    id: UUID = ApiField(alias='id')
    name: str = ApiField(alias='name')

class ConfigurationInformation(ObjectModel):
    clientAuthentication: ClientAuthenticationInformation = ApiField(alias='clientAuthentication')
    cloudProviders: CloudProvidersInformation = ApiField(alias='cloudProviders')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: str = ApiField(alias='creationDate')
    id: UUID = ApiField(alias='id')
    longLivedCertCount: int = ApiField(alias='longLivedCertCount')
    modificationDate: str = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    serviceAccountIds: List[UUID] = ApiField(alias='serviceAccountIds', default_factory=list)
    shortLivedCertCount: int = ApiField(alias='shortLivedCertCount')
    ultraShortLivedCertCount: int = ApiField(alias='ultraShortLivedCertCount')

class ConfigurationResponse(ObjectModel):
    configurations: List[ExtendedConfigurationInformation] = ApiField(alias='configurations', default_factory=list)

class ConfigurationUpdateRequest(ObjectModel):
    clientAuthentication: ClientAuthenticationRequestOpenApi = ApiField(alias='clientAuthentication')
    cloudProviders: CloudProvidersInformation = ApiField(alias='cloudProviders')
    name: str = ApiField(alias='name')
    policyIds: UUID = ApiField(alias='policyIds')
    serviceAccountIds: List[UUID] = ApiField(alias='serviceAccountIds', default_factory=list)
    subCaProviderId: UUID = ApiField(alias='subCaProviderId')

class ErrorInformation(ObjectModel):
    args: List[AnyValue] = ApiField(alias='args', default_factory=list)
    code: int = ApiField(alias='code')
    message: str = ApiField(alias='message')

class ErrorResponse(ObjectModel):
    errors: List[ErrorInformation] = ApiField(alias='errors', default_factory=list)

class ExtendedConfigurationInformation(ObjectModel):
    clientAuthentication: ClientAuthenticationInformation = ApiField(alias='clientAuthentication')
    cloudProviders: CloudProvidersInformation = ApiField(alias='cloudProviders')
    companyId: UUID = ApiField(alias='companyId')
    creationDate: str = ApiField(alias='creationDate')
    id: UUID = ApiField(alias='id')
    longLivedCertCount: int = ApiField(alias='longLivedCertCount')
    modificationDate: str = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    policies: List[PolicyInformation] = ApiField(alias='policies', default_factory=list)
    serviceAccountIds: List[UUID] = ApiField(alias='serviceAccountIds', default_factory=list)
    shortLivedCertCount: int = ApiField(alias='shortLivedCertCount')
    subCaProvider: SubCaProviderInformation = ApiField(alias='subCaProvider')
    ultraShortLivedCertCount: int = ApiField(alias='ultraShortLivedCertCount')

class ExtendedPolicyInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    configurations: List[ConfigurationInformation] = ApiField(alias='configurations', default_factory=list)
    creationDate: str = ApiField(alias='creationDate')
    extendedKeyUsages: Literal[
        'ANY', 'CAPWAP_AC', 'CAPWAP_WTP', 'CLIENT_AUTH', 'CODE_SIGNING', 'DVCS', 'EAP_OVER_LAN', 'EAP_OVER_PPP', 'EMAIL_PROTECTION', 'IPSEC_ENDSYSTEM', 'IPSEC_IKE', 'IPSEC_IKE_INTERMEDIATE',
        'IPSEC_TUNNEL', 'IPSEC_USER', 'OCSP_SIGNING', 'SBGP_CERT_AA_SERVER_AUTH', 'SCVP_CLIENT', 'SCVP_RESPONDER', 'SCVP_SERVER', 'SERVER_AUTH', 'SMARTCARD_LOGON', 'TIME_STAMPING'] = ApiField(
        alias='extendedKeyUsages'
    )
    id: UUID = ApiField(alias='id')
    keyAlgorithm: KeyAlgorithmInformation = ApiField(alias='keyAlgorithm')
    keyUsages: Literal['cRLSign', 'dataEncipherment', 'decipherOnly', 'digitalSignature', 'encipherOnly',
    'keyAgreement', 'keyCertSign', 'keyEncipherment', 'nonRepudiation'] = ApiField(alias='keyUsages')
    modificationDate: str = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    sans: SansInformation = ApiField(alias='sans')
    subject: SubjectAttributesInformation = ApiField(alias='subject')
    validityPeriod: str = ApiField(alias='validityPeriod')

class GoogleCloudProviderInformation(ObjectModel):
    projectIdentifiers: List[str] = ApiField(alias='projectIdentifiers', default_factory=list)
    regions: List[Literal[
        'asia-east1', 'asia-east2', 'asia-northeast1', 'asia-northeast2', 'asia-northeast3', 'asia-south1', 'asia-south2', 'asia-southeast1', 'asia-southeast2', 'australia-southeast1', 'australia-southeast2', 'europe-central2', 'europe-north1', 'europe-southwest1', 'europe-west1', 'europe-west12', 'europe-west2', 'europe-west3',
        'europe-west4', 'europe-west6', 'europe-west8', 'europe-west9', 'me-central1', 'me-west1', 'northamerica-northeast1', 'northamerica-northeast2', 'southamerica-east1', 'southamerica-west1', 'us-central1', 'us-east1', 'us-east4', 'us-east5', 'us-south1', 'us-west1', 'us-west2', 'us-west3', 'us-west4']] = ApiField(
        alias='regions',
        default_factory=list
    )

class IntermediateCertificateInformation(ObjectModel):
    certificate: str = ApiField(alias='certificate')
    commonName: str = ApiField(alias='commonName')
    companyId: UUID = ApiField(alias='companyId')
    configuration: ConfigurationInformation = ApiField(alias='configuration')
    creationDate: str = ApiField(alias='creationDate')
    errorInformation: ErrorInformation = ApiField(alias='errorInformation')
    fingerprint: str = ApiField(alias='fingerprint')
    id: UUID = ApiField(alias='id')
    issuerCertificates: List[str] = ApiField(alias='issuerCertificates', default_factory=list)
    longLivedCertCount: int = ApiField(alias='longLivedCertCount')
    modificationDate: str = ApiField(alias='modificationDate')
    shortLivedCertCount: int = ApiField(alias='shortLivedCertCount')
    status: str = ApiField(alias='status')
    ultraShortLivedCertCount: int = ApiField(alias='ultraShortLivedCertCount')
    validityEnd: str = ApiField(alias='validityEnd')
    validityStart: str = ApiField(alias='validityStart')
    workflowId: str = ApiField(alias='workflowId')

class IntermediateCertificateResponse(ObjectModel):
    intermediateCertificates: List[IntermediateCertificateInformation] = ApiField(
        alias='intermediateCertificates',
        default_factory=list
    )

class JwtJwksAuthenticationInformation(ClientAuthenticationInformation):
    urls: List[str] = ApiField(alias='urls', default_factory=list)

class JwtJwksAuthenticationOpenApi(ClientAuthenticationOpenApi):
    urls: List[str] = ApiField(alias='urls', default_factory=list)

class JwtOidcAuthenticationInformation(ClientAuthenticationInformation):
    audience: str = ApiField(alias='audience')
    baseUrl: str = ApiField(alias='baseUrl')

class JwtOidcAuthenticationOpenApi(ClientAuthenticationOpenApi):
    audience: str = ApiField(alias='audience')
    baseUrl: str = ApiField(alias='baseUrl')

class KeyAlgorithmInformation(ObjectModel):
    allowedValues: Literal[
        'EC_ED25519', 'EC_P256', 'EC_P384', 'EC_P521', 'RSA_2048', 'RSA_3072', 'RSA_4096'] = ApiField(
        alias='allowedValues'
    )
    defaultValue: Literal['EC_ED25519', 'EC_P256', 'EC_P384', 'EC_P521', 'RSA_2048', 'RSA_3072', 'RSA_4096'] = ApiField(
        alias='defaultValue'
    )

class PolicyCreateRequest(ObjectModel):
    extendedKeyUsages: Literal[
        'ANY', 'CAPWAP_AC', 'CAPWAP_WTP', 'CLIENT_AUTH', 'CODE_SIGNING', 'DVCS', 'EAP_OVER_LAN', 'EAP_OVER_PPP', 'EMAIL_PROTECTION', 'IPSEC_ENDSYSTEM', 'IPSEC_IKE', 'IPSEC_IKE_INTERMEDIATE',
        'IPSEC_TUNNEL', 'IPSEC_USER', 'OCSP_SIGNING', 'SBGP_CERT_AA_SERVER_AUTH', 'SCVP_CLIENT', 'SCVP_RESPONDER', 'SCVP_SERVER', 'SERVER_AUTH', 'SMARTCARD_LOGON', 'TIME_STAMPING'] = ApiField(
        alias='extendedKeyUsages'
    )
    keyAlgorithm: KeyAlgorithmInformation = ApiField(alias='keyAlgorithm')
    keyUsages: Literal['cRLSign', 'dataEncipherment', 'decipherOnly', 'digitalSignature', 'encipherOnly',
    'keyAgreement', 'keyCertSign', 'keyEncipherment', 'nonRepudiation'] = ApiField(alias='keyUsages')
    name: str = ApiField(alias='name')
    sans: SansInformation = ApiField(alias='sans')
    subject: SubjectAttributesInformation = ApiField(alias='subject')
    validityPeriod: str = ApiField(alias='validityPeriod')

class PolicyDeleteResponse(ObjectModel):
    id: UUID = ApiField(alias='id')
    name: str = ApiField(alias='name')

class PolicyInformation(ObjectModel):
    companyId: UUID = ApiField(alias='companyId')
    creationDate: str = ApiField(alias='creationDate')
    extendedKeyUsages: Literal[
        'ANY', 'CAPWAP_AC', 'CAPWAP_WTP', 'CLIENT_AUTH', 'CODE_SIGNING', 'DVCS', 'EAP_OVER_LAN', 'EAP_OVER_PPP', 'EMAIL_PROTECTION', 'IPSEC_ENDSYSTEM', 'IPSEC_IKE', 'IPSEC_IKE_INTERMEDIATE',
        'IPSEC_TUNNEL', 'IPSEC_USER', 'OCSP_SIGNING', 'SBGP_CERT_AA_SERVER_AUTH', 'SCVP_CLIENT', 'SCVP_RESPONDER', 'SCVP_SERVER', 'SERVER_AUTH', 'SMARTCARD_LOGON', 'TIME_STAMPING'] = ApiField(
        alias='extendedKeyUsages'
    )
    id: UUID = ApiField(alias='id')
    keyAlgorithm: KeyAlgorithmInformation = ApiField(alias='keyAlgorithm')
    keyUsages: Literal['cRLSign', 'dataEncipherment', 'decipherOnly', 'digitalSignature', 'encipherOnly',
    'keyAgreement', 'keyCertSign', 'keyEncipherment', 'nonRepudiation'] = ApiField(alias='keyUsages')
    modificationDate: str = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    sans: SansInformation = ApiField(alias='sans')
    subject: SubjectAttributesInformation = ApiField(alias='subject')
    validityPeriod: str = ApiField(alias='validityPeriod')

class PolicyResponse(ObjectModel):
    policies: List[ExtendedPolicyInformation] = ApiField(alias='policies', default_factory=list)

class PolicyUpdateRequest(ObjectModel):
    extendedKeyUsages: Literal[
        'ANY', 'CAPWAP_AC', 'CAPWAP_WTP', 'CLIENT_AUTH', 'CODE_SIGNING', 'DVCS', 'EAP_OVER_LAN', 'EAP_OVER_PPP', 'EMAIL_PROTECTION', 'IPSEC_ENDSYSTEM', 'IPSEC_IKE', 'IPSEC_IKE_INTERMEDIATE',
        'IPSEC_TUNNEL', 'IPSEC_USER', 'OCSP_SIGNING', 'SBGP_CERT_AA_SERVER_AUTH', 'SCVP_CLIENT', 'SCVP_RESPONDER', 'SCVP_SERVER', 'SERVER_AUTH', 'SMARTCARD_LOGON', 'TIME_STAMPING'] = ApiField(
        alias='extendedKeyUsages'
    )
    keyAlgorithm: KeyAlgorithmInformation = ApiField(alias='keyAlgorithm')
    keyUsages: Literal['cRLSign', 'dataEncipherment', 'decipherOnly', 'digitalSignature', 'encipherOnly',
    'keyAgreement', 'keyCertSign', 'keyEncipherment', 'nonRepudiation'] = ApiField(alias='keyUsages')
    name: str = ApiField(alias='name')
    sans: SansInformation = ApiField(alias='sans')
    subject: SubjectAttributesInformation = ApiField(alias='subject')
    validityPeriod: str = ApiField(alias='validityPeriod')

class PropertyInformation(ObjectModel):
    allowedValues: List[str] = ApiField(alias='allowedValues', default_factory=list)
    defaultValues: List[str] = ApiField(alias='defaultValues', default_factory=list)
    maxOccurrences: int = ApiField(alias='maxOccurrences')
    minOccurrences: int = ApiField(alias='minOccurrences')
    type: Literal['FORBIDDEN', 'IGNORED', 'LOCKED', 'OPTIONAL', 'REQUIRED'] = ApiField(alias='type')

class SansInformation(ObjectModel):
    dnsNames: PropertyInformation = ApiField(alias='dnsNames')
    ipAddresses: PropertyInformation = ApiField(alias='ipAddresses')
    rfc822Names: PropertyInformation = ApiField(alias='rfc822Names')
    uniformResourceIdentifiers: PropertyInformation = ApiField(alias='uniformResourceIdentifiers')

class SubCaProviderCreateRequest(ObjectModel):
    caAccountId: UUID = ApiField(alias='caAccountId')
    caProductOptionId: UUID = ApiField(alias='caProductOptionId')
    caType: Literal['ACME', 'BUILTIN', 'DIGICERT', 'ENTRUST', 'GLOBALSIGN',
    'GLOBALSIGNMSSL', 'MICROSOFT', 'MOCKCA', 'TPP', 'ZTPKI'] = ApiField(alias='caType')
    commonName: str = ApiField(alias='commonName')
    country: str = ApiField(alias='country')
    keyAlgorithm: Literal['EC_ED25519', 'EC_P256', 'EC_P384', 'EC_P521', 'RSA_2048', 'RSA_3072', 'RSA_4096'] = ApiField(
        alias='keyAlgorithm'
    )
    locality: str = ApiField(alias='locality')
    name: str = ApiField(alias='name')
    organization: str = ApiField(alias='organization')
    organizationalUnit: str = ApiField(alias='organizationalUnit')
    pkcs11: SubCaProviderPkcs11ConfigurationInformation = ApiField(alias='pkcs11')
    stateOrProvince: str = ApiField(alias='stateOrProvince')
    validityPeriod: str = ApiField(alias='validityPeriod')

class SubCaProviderDeleteResponse(ObjectModel):
    id: UUID = ApiField(alias='id')
    name: str = ApiField(alias='name')

class SubCaProviderInformation(ObjectModel):
    caAccountId: UUID = ApiField(alias='caAccountId')
    caProductOptionId: UUID = ApiField(alias='caProductOptionId')
    caType: Literal['ACME', 'BUILTIN', 'DIGICERT', 'ENTRUST', 'GLOBALSIGN',
    'GLOBALSIGNMSSL', 'MICROSOFT', 'MOCKCA', 'TPP', 'ZTPKI'] = ApiField(alias='caType')
    commonName: str = ApiField(alias='commonName')
    companyId: UUID = ApiField(alias='companyId')
    country: str = ApiField(alias='country')
    creationDate: str = ApiField(alias='creationDate')
    id: UUID = ApiField(alias='id')
    keyAlgorithm: Literal['EC_ED25519', 'EC_P256', 'EC_P384', 'EC_P521', 'RSA_2048', 'RSA_3072', 'RSA_4096'] = ApiField(
        alias='keyAlgorithm'
    )
    locality: str = ApiField(alias='locality')
    modificationDate: str = ApiField(alias='modificationDate')
    name: str = ApiField(alias='name')
    organization: str = ApiField(alias='organization')
    organizationalUnit: str = ApiField(alias='organizationalUnit')
    pkcs11: SubCaProviderPkcs11ConfigurationInformation = ApiField(alias='pkcs11')
    stateOrProvince: str = ApiField(alias='stateOrProvince')
    validityPeriod: str = ApiField(alias='validityPeriod')

class SubCaProviderPkcs11ConfigurationInformation(ObjectModel):
    allowedClientLibraries: List[str] = ApiField(alias='allowedClientLibraries', default_factory=list)
    pin: str = ApiField(alias='pin')
    signingEnabled: bool = ApiField(alias='signingEnabled')
    slot: int = ApiField(alias='slot')

class SubCaProviderResponse(ObjectModel):
    subCaProviders: List[SubCaProviderInformation] = ApiField(alias='subCaProviders', default_factory=list)

class SubCaProviderUpdateRequest(ObjectModel):
    caProductOptionId: UUID = ApiField(alias='caProductOptionId')
    commonName: str = ApiField(alias='commonName')
    country: str = ApiField(alias='country')
    keyAlgorithm: Literal['EC_ED25519', 'EC_P256', 'EC_P384', 'EC_P521', 'RSA_2048', 'RSA_3072', 'RSA_4096'] = ApiField(
        alias='keyAlgorithm'
    )
    locality: str = ApiField(alias='locality')
    name: str = ApiField(alias='name')
    organization: str = ApiField(alias='organization')
    organizationalUnit: str = ApiField(alias='organizationalUnit')
    pkcs11: SubCaProviderPkcs11ConfigurationInformation = ApiField(alias='pkcs11')
    stateOrProvince: str = ApiField(alias='stateOrProvince')
    validityPeriod: str = ApiField(alias='validityPeriod')

class SubjectAttributesInformation(ObjectModel):
    commonName: PropertyInformation = ApiField(alias='commonName')
    country: PropertyInformation = ApiField(alias='country')
    locality: PropertyInformation = ApiField(alias='locality')
    organization: PropertyInformation = ApiField(alias='organization')
    organizationalUnit: PropertyInformation = ApiField(alias='organizationalUnit')
    stateOrProvince: PropertyInformation = ApiField(alias='stateOrProvince')

AwsCloudProviderInformation.update_forward_refs()
AzureCloudProviderInformation.update_forward_refs()
ClientAuthenticationInformation.update_forward_refs()
ClientAuthenticationOpenApi.update_forward_refs()
ClientAuthenticationRequestOpenApi.update_forward_refs()
CloudProvidersInformation.update_forward_refs()
ConfigurationCreateRequest.update_forward_refs()
ConfigurationDeleteResponse.update_forward_refs()
ConfigurationInformation.update_forward_refs()
ConfigurationResponse.update_forward_refs()
ConfigurationUpdateRequest.update_forward_refs()
ErrorInformation.update_forward_refs()
ErrorResponse.update_forward_refs()
ExtendedConfigurationInformation.update_forward_refs()
ExtendedPolicyInformation.update_forward_refs()
GoogleCloudProviderInformation.update_forward_refs()
IntermediateCertificateInformation.update_forward_refs()
IntermediateCertificateResponse.update_forward_refs()
JwtJwksAuthenticationInformation.update_forward_refs()
JwtJwksAuthenticationOpenApi.update_forward_refs()
JwtOidcAuthenticationInformation.update_forward_refs()
JwtOidcAuthenticationOpenApi.update_forward_refs()
KeyAlgorithmInformation.update_forward_refs()
PolicyCreateRequest.update_forward_refs()
PolicyDeleteResponse.update_forward_refs()
PolicyInformation.update_forward_refs()
PolicyResponse.update_forward_refs()
PolicyUpdateRequest.update_forward_refs()
PropertyInformation.update_forward_refs()
SansInformation.update_forward_refs()
SubCaProviderCreateRequest.update_forward_refs()
SubCaProviderDeleteResponse.update_forward_refs()
SubCaProviderInformation.update_forward_refs()
SubCaProviderPkcs11ConfigurationInformation.update_forward_refs()
SubCaProviderResponse.update_forward_refs()
SubCaProviderUpdateRequest.update_forward_refs()
SubjectAttributesInformation.update_forward_refs()



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.branch_base import BranchBaseAttributes

class DiscoveryRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Discovery Root"
    address_range = Attribute('Address Range', min_version='21.4')
    dsn = Attribute('DSN', min_version='21.4')
    driver_name = Attribute('Driver Name', min_version='21.4')
    protection_key = Attribute('Protection Key', min_version='21.4')
    timeout = Attribute('Timeout', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class CertificateTrustBundleAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Certificate Trust Bundle"
    excluded_certificates = Attribute('Excluded Certificates')
    included_certificates = Attribute('Included Certificates')
    is_match = Attribute('Is Match')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.encryption_driver import EncryptionDriverAttributes

class DPAPIEncryptionDriverAttributes(EncryptionDriverAttributes, metaclass=IterableMeta):
    __config_class__ = "DPAPI Encryption Driver"
    generation_only = Attribute('Generation Only', min_version='21.4')
    key_validation = Attribute('Key Validation', min_version='21.4')
    verigram = Attribute('VeriGram', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)

class Output(ObjectModel):
    success: bool = ApiField(alias='Success')
    error_code: int = ApiField(alias='ErrorCode')
    error_message: str = ApiField(alias='ErrorMessage')

class ProcessingDetails(ObjectModel):
    status: str = ApiField(alias='Status')
    status_description: str = ApiField(alias='StatusDescription')

class CertificateDetails(ObjectModel):
    ca_fingerprint_sha256: str = ApiField(alias='CaFingerprintSha256')
    certificate_fingerprint_sha256: str = ApiField(alias='CertificateFingerprintSha256')
    certificate_type: str = ApiField(alias='CertificateType')
    extensions: dict = ApiField(alias='Extensions')
    key_id: str = ApiField(alias='KeyId')
    key_type: str = ApiField(alias='KeyType')
    principals: list[str] = ApiField(alias='Principals', default_factory=list)
    public_key_fingerprint_sha256: str = ApiField(alias='PublicKeyFingerprintSha256')
    serial_number: str = ApiField(alias='SerialNumber')
    source_addresses: list[str] = ApiField(alias='SourceAddresses', default_factory=list)
    valid_from: int = ApiField(alias='ValidFrom')
    valid_to: int = ApiField(alias='ValidTo')

class RequestDetails(ObjectModel):
    originating_ip: str = ApiField(alias='OriginatingIp')
    requested_by: str = ApiField(alias='RequestedBy')

class AccessControl(ObjectModel):
    allowed_certificate_identifier_patterns: list[str] = ApiField(
        alias='AllowedCertificateIdentifierPatterns',
        default_factory=list
    )
    allowed_extensions: list[str] = ApiField(alias='AllowedExtensions', default_factory=list)
    allowed_force_command_patterns: list[str] = ApiField(alias='AllowedForceCommandPatterns', default_factory=list)
    allowed_principals_patterns: list[str] = ApiField(alias='AllowedPrincipalsPatterns', default_factory=list)
    allowed_source_addresses: list[str] = ApiField(alias='AllowedSourceAddresses', default_factory=list)
    default_certificate_identifier: str = ApiField(alias='DefaultCertificateIdentifier')
    default_extensions: list[str] = ApiField(alias='DefaultExtensions', default_factory=list)
    default_force_command: str = ApiField(alias='DefaultForceCommand')
    default_principals: list[str] = ApiField(alias='DefaultPrincipals', default_factory=list)
    default_source_addresses: list[str] = ApiField(alias='DefaultSourceAddresses', default_factory=list)

class APIClient(ObjectModel):
    allowed_to_request_certificate_identifier: bool = ApiField(alias='AllowedToRequestCertificateIdentifier')
    allowed_to_request_extensions: bool = ApiField(alias='AllowedToRequestExtensions')
    allowed_to_request_force_command: bool = ApiField(alias='AllowedToRequestForceCommand')
    allowed_to_request_principals: bool = ApiField(alias='AllowedToRequestPrincipals')
    allowed_to_request_source_addresses: bool = ApiField(alias='AllowedToRequestSourceAddresses')

class Certificate(ObjectModel):
    allowed_private_key_algorithms: list[str] = ApiField(alias='AllowedPrivateKeyAlgorithms', default_factory=list)
    allowed_private_key_reuse: bool = ApiField(alias='AllowedPrivateKeyReuse')
    certificate_destination_dn: str = ApiField(alias='CertificateDestinationDn')
    default_private_key_algorithm: str = ApiField(alias='DefaultPrivateKeyAlgorithm')
    signature_hashing_algorithm: str = ApiField(alias='SignatureHashingAlgorithm')
    type: str = ApiField(alias='Type')
    validity_period: str = ApiField(alias='ValidityPeriod')

class CAKeyPair(ObjectModel):
    created_on: str = ApiField(alias='CreatedOn')
    dn: str = ApiField(alias='Dn')
    fingerprint_sha256: str = ApiField(alias='FingerprintSha256')
    guid: str = ApiField(alias='Guid')
    key_algorithm: str = ApiField(alias='KeyAlgorithm')
    name: str = ApiField(alias='Name')
    public_key_data: str = ApiField(alias='PublicKeyData')

class AvailableTemplate(ObjectModel):
    dn: str = ApiField(alias='DN')
    guid: str = ApiField(alias='Guid')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.credential_base import CredentialBaseAttributes

class GenerationalCredentialAttributes(CredentialBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Generational Credential"
    last = Attribute('Last', min_version='21.4')
    visibility = Attribute('Visibility', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.report_base import ReportBaseAttributes

class ReportHistoricalCertificatesAttributes(ReportBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Report:HistoricalCertificates"
    options = Attribute('Options', min_version='23.3')
    policydn = Attribute('PolicyDN', min_version='23.3')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.driver_base import DriverBaseAttributes

class CloudInstanceMonitorDriverBaseAttributes(DriverBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "Cloud Instance Monitor Driver Base"
    certificate_cleanup_options = Attribute('Certificate Cleanup Options')
    certificate_relocation_policy_dn = Attribute('Certificate Relocation Policy DN')
    cleanup_after = Attribute('Cleanup After')
    cloud_region = Attribute('Cloud Region')
    last_run = Attribute('Last Run')
    policydn = Attribute('PolicyDN')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.application_base import ApplicationBaseAttributes

class F5LTMAdvancedAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
    __config_class__ = "F5 LTM Advanced"
    advanced_settings_bundle_name = Attribute('Advanced Settings Bundle Name', min_version='21.4')
    advertised_ca = Attribute('Advertised CA', min_version='21.4')
    archive_location = Attribute('Archive Location', min_version='21.4')
    associate_ssl_profile_to = Attribute('Associate SSL Profile To', min_version='22.2')
    authentication_frequency = Attribute('Authentication Frequency', min_version='21.4')
    build = Attribute('Build', min_version='21.4')
    bundle_certificate = Attribute('Bundle Certificate', min_version='21.4')
    bundle_certificate_collection = Attribute('Bundle Certificate Collection', min_version='21.4')
    crl = Attribute('CRL', min_version='21.4')
    certificate_chain_name = Attribute('Certificate Chain Name', min_version='21.4')
    certificate_name = Attribute('Certificate Name', min_version='21.4')
    chain_traversal_depth = Attribute('Chain Traversal Depth', min_version='21.4')
    client_authentication_certificate = Attribute('Client Authentication Certificate', min_version='21.4')
    config_sync = Attribute('Config Sync', min_version='21.4')
    connection_attempts = Attribute('Connection Attempts', min_version='21.4')
    delete_previous_cert_and_key = Attribute('Delete Previous Cert And Key', min_version='21.4')
    device_certificate = Attribute('Device Certificate', min_version='21.4')
    file_validation_disabled = Attribute('File Validation Disabled', min_version='21.4')
    fips_key = Attribute('Fips Key', min_version='21.4')
    force_profile_update = Attribute('Force Profile Update', min_version='21.4')
    install_chain_file = Attribute('Install Chain File', min_version='21.4')
    last_used_host = Attribute('Last Used Host', min_version='21.4')
    monitor_name = Attribute('Monitor Name', min_version='22.2')
    monitor_partition = Attribute('Monitor Partition', min_version='22.2')
    network_validation_disabled = Attribute('Network Validation Disabled', min_version='21.4')
    overwrite_certificate = Attribute('Overwrite Certificate', min_version='21.4')
    overwrite_existing_chain = Attribute('Overwrite Existing Chain', min_version='21.4')
    parent_ssl_profile_name = Attribute('Parent SSL Profile Name', min_version='21.4')
    partition = Attribute('Partition', min_version='21.4')
    previous_certificate = Attribute('Previous Certificate', min_version='21.4')
    previous_key = Attribute('Previous Key', min_version='21.4')
    provisioning_to = Attribute('Provisioning To', min_version='21.4')
    sni_default = Attribute('SNI Default', min_version='21.4')
    sni_server_name = Attribute('SNI Server Name', min_version='21.4')
    ssh_port = Attribute('SSH Port', min_version='21.4')
    ssl_profile_name = Attribute('SSL Profile Name', min_version='21.4')
    ssl_profile_type = Attribute('SSL Profile Type', min_version='21.4')
    server_authentication_certificate = Attribute('Server Authentication Certificate', min_version='21.4')
    server_authentication_name = Attribute('Server Authentication Name', min_version='21.4')
    system_id = Attribute('System Id', min_version='21.4')
    trusted_ca = Attribute('Trusted CA', min_version='21.4')
    use_advanced_settings = Attribute('Use Advanced Settings', min_version='21.4')
    use_basic_provisioning = Attribute('Use Basic Provisioning', min_version='21.4')
    use_rest_api = Attribute('Use REST API', min_version='21.4')
    version = Attribute('Version', min_version='21.4')
    virtual_server_name = Attribute('Virtual Server Name', min_version='21.4')
    virtual_server_partition = Attribute('Virtual Server Partition', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class CredentialBaseAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Credential Base"
    escalation_notice_interval = Attribute('Escalation Notice Interval')
    escalation_notice_start = Attribute('Escalation Notice Start')
    expiration = Attribute('Expiration')
    expiration_notice_interval = Attribute('Expiration Notice Interval')
    expiration_notice_start = Attribute('Expiration Notice Start')
    last_notification = Attribute('Last Notification')
    notified_on = Attribute('Notified On')
    protection_key = Attribute('Protection Key')
    shared = Attribute('Shared')
    vault_id = Attribute('Vault Id')



from __future__ import annotations

from pyvenafi.tpp.api.api_base import (
    ApiField,
    ObjectModel,
)
from pyvenafi.tpp.api.websdk.models.resultcodes import ResultCodes

class Result(ObjectModel):
    code: int = ApiField()

    @property
    def metadata_result(self) -> str:
        return ResultCodes.Metadata.get(self.code, 'Unknown')

class Item(ObjectModel):
    allowed_characters: str = ApiField(alias='AllowedCharacters')
    allowed_values: str = ApiField(alias='AllowedValues')
    category: str = ApiField(alias='Category')
    classes: list = ApiField(alias='Classes')
    config_attribute: str = ApiField(alias='ConfigAttribute')
    date_only: bool = ApiField(alias='DateOnly')
    default_values: str = ApiField(alias='DefaultValues')
    display_after: str = ApiField(alias='DisplayAfter')
    dn: str = ApiField(alias='Dn')
    error_message: str = ApiField(alias='ErrorMessage')
    guid: str = ApiField(alias='Guid')
    help: str = ApiField(alias='Help')
    label: str = ApiField(alias='Label')
    localization_table: str = ApiField(alias='LocalizationTable')
    localized_help: str = ApiField(alias='LocalizedHelp')
    localized_label: str = ApiField(alias='LocalizedLabel')
    localized_set: str = ApiField(alias='LocalizedSet')
    mandatory: bool = ApiField(alias='Mandatory')
    name: str = ApiField(alias='Name')
    mask: str = ApiField(alias='Mask')
    maximum_length: int = ApiField(alias='MaximumLength')
    minimum_length: int = ApiField(alias='MinimumLength')
    policyable: bool = ApiField(alias='Policyable')
    regular_expression: str = ApiField(alias='RegularExpression')
    render_hidden: bool = ApiField(alias='RenderHidden')
    render_read_only: bool = ApiField(alias='RenderReadOnly')
    single: bool = ApiField(alias='Single')
    time_only: bool = ApiField(alias='TimeOnly')
    type: int = ApiField(alias='Type')

class Data(ObjectModel):
    key: Item = ApiField(alias='Key')
    value: list[str] = ApiField(alias='Value', default_factory=list)

class PolicyItem(ObjectModel):
    key: str = ApiField(alias='Key')
    value: list[Item] = ApiField(alias='Value', default_factory=list)

class NameValue(ObjectModel):
    name: str = ApiField(alias='Name')
    value: str = ApiField(alias='Value')

class Update(ObjectModel):
    dn: str = ApiField(alias='DN')
    data: list[NameValue] = ApiField(alias='Data')

class GuidData(ObjectModel):
    item_guid: str = ApiField(alias='ItemGuid')
    list: list[str] = ApiField(alias='List')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.agent_base import AgentBaseAttributes
from pyvenafi.tpp.attributes.driver_base import DriverBaseAttributes
from pyvenafi.tpp.attributes.legacy_key_base import LegacyKeyBaseAttributes
from pyvenafi.tpp.attributes.monitoring_base import MonitoringBaseAttributes

class SSHKeyAttributes(
    AgentBaseAttributes,
    DriverBaseAttributes,
    LegacyKeyBaseAttributes,
    MonitoringBaseAttributes,
    metaclass=IterableMeta
):
    __config_class__ = "SSH Key"
    public_key_vault_id = Attribute('Public Key Vault Id', min_version='21.4')



from __future__ import annotations

from pyvenafi.tpp.attributes._helper import (
    IterableMeta,
    Attribute,
)
from pyvenafi.tpp.attributes.top import TopAttributes

class CodeSigningEnvironmentBaseAttributes(TopAttributes, metaclass=IterableMeta):
    __config_class__ = "Code Signing Environment Base"
    change_management_comment = Attribute('Change Management Comment')
    change_management_entry = Attribute('Change Management Entry')
    change_management_flow_identifier = Attribute('Change Management Flow Identifier')
    change_management_status = Attribute('Change Management Status')
    friendly_name = Attribute('Friendly Name')
    ip_address_restriction = Attribute('IP Address Restriction')
    key_time_constraint = Attribute('Key Time Constraint')
    key_use_authentication = Attribute('Key Use Authentication')
    key_use_flow_dn = Attribute('Key Use Flow DN')
    per_user = Attribute('Per User')
    prevent_use_of_expired_environment = Attribute('Prevent Use Of Expired Environment')
    related_items_deleted_along_with_this_environment = Attribute('Related Items Deleted Along With This Environment')
    require_justification = Attribute('Require Justification')
    status = Attribute('Status')
    status_message = Attribute('Status Message')
    template_dn = Attribute('Template DN')
    user_key_import = Attribute('User Key Import')
